# * coding: utf8 *


try:
    from .utils import trans_date
except:
    from chncal.utils import trans_date
    
    
XINGZUO = {
    '摩羯座': {
        'dt_rng': ('1222', '0119'),
        'desc': '摩羯座的人以稳重、务实、有责任感著称。他们注重事业和成就，有很强的自律性，但有时会显得过于严肃或缺乏灵活性。'
        },
    '水瓶座': {
        'dt_rng': ('0120', '0218'),
        'desc': '水瓶座的人以独立、创新、人道主义著称。他们思想开放，善于接受新观念，追求自由和平等。水瓶座的人通常性格开朗，但有时会显得冷漠或过于理性。'
        },
    '双鱼座': {
        'dt_rng': ('0219', '0320'),
        'desc': '双鱼座的人以温柔、敏感、富有同情心著称。他们富有创造力，喜欢沉浸在自己的世界中，但有时会显得过于理想化或容易受他人影响。'
        },
    '白羊座': {
        'dt_rng':  ('0321', '0419'),
        'desc': '白羊座是黄道十二星座的第一个星座，象征着新的开始。白羊座的人通常充满活力、热情、勇敢，喜欢冒险和挑战。他们性格直爽，行动力强，但有时可能会显得冲动和缺乏耐心。'
        },
    '金牛座': {
        'dt_rng': ('0420', '0520'),
        'desc': '金牛座的人以稳重、踏实著称。他们热爱生活，注重物质享受，对美好事物有很高的追求。金牛座的人通常耐心十足，但有时会显得固执，不喜欢改变。'
        },
    '双子座': {
        'dt_rng': ('0521', '0621'),
        'desc': '双子座是风象星座，以聪明、灵活、善于沟通著称。双子座的人好奇心旺盛，喜欢探索新事物，思维敏捷，但有时会显得善变，难以捉摸。'
        },
    '巨蟹座': {
        'dt_rng': ('0622', '0722'),
        'desc': '巨蟹座的人以家庭为中心，情感丰富，富有同情心。他们善良、体贴，对家人和朋友非常忠诚，但有时会显得情绪化，对外界的批评比较敏感。'
        },
    '狮子座': {
        'dt_rng': ('0723', '0822'),
        'desc': '狮子座的人天生具有领导才能，自信、热情、充满魅力。他们喜欢成为众人关注的焦点，有很强的责任感，但有时会显得自负或过于自我中心。'
        },
    '处女座': {
        'dt_rng': ('0823', '0922'),
        'desc': '处女座的人以完美主义著称，注重细节，追求高品质的生活。他们聪明、勤奋、有条理，但有时会显得过于挑剔或过于关注细节而忽略了整体。'
        },
    '天秤座': {
        'dt_rng': ('0923', '1023'),
        'desc': '天秤座的人追求和谐与平衡，善于社交，具有很强的审美能力。他们性格温和，善于调解冲突，但有时会显得优柔寡断，难以做出决策。'
        },
    '天蝎座': {
        'dt_rng': ('1023', '1122'),
        'desc': '天蝎座的人以神秘、热情、坚定著称。他们洞察力强，善于隐藏自己的情感，对事物有深刻的思考。天蝎座的人忠诚、专注，但有时会显得占有欲强或过于敏感。'
        },
    '射手座': {
        'dt_rng': ('1123', '1221'),
        'desc': '射手座的人热爱自由，乐观开朗，喜欢冒险和探索新事物。他们性格直率，善于沟通，但有时会显得缺乏耐心或过于冲动。'
        }
}


def get_xingzuo(date=None, with_desc=False, is_md=False):
    """通过日期获取星座，若is_md=True，则date格式须形如: `0514`"""
    if not is_md:
        date = trans_date(date)
        md = date.strftime('%m%d')
    else:
        md = date
    name_ = None
    for name, info in XINGZUO.items():
        if md >= info['dt_rng'][0] and md <= info['dt_rng'][1]:
            name_ = name
    if name_ is None:
        name_ = '摩羯座'
    if not with_desc:
        return name_
    else:
        return name_, XINGZUO[name_]['desc']




# -*- coding: utf-8 -*-

import os
import time
import datetime
import yaml
from functools import wraps
import pandas as pd


def get_file_path(fname, dirname='constants'):
    cur_dir = os.path.dirname(os.path.realpath(__file__))
    return os.path.join(cur_dir, dirname, fname)    


def _load_yml(fpath, **kwargs_open):
    with open(fpath, **kwargs_open) as f:
        data = yaml.load(f.read(), Loader=yaml.FullLoader)
    return data


def load_yml(*args, **kwargs):
    if 'encoding' not in kwargs:
        for en in ['utf-8', None, 'gbk']:
            try:
                kwargs.update({'encoding': en})
                return _load_yml(*args, **kwargs)
            except:
                pass
    return _load_yml(*args, **kwargs)


def trans_date(date):
    if pd.isnull(date):
        date = datetime.datetime.now()
    if isinstance(date, str):
        date = pd.to_datetime(date)
    elif isinstance(date, int):
        date = pd.to_datetime(str(date))
    elif isinstance(date, time.struct_time):
        date = pd.to_datetime(
               datetime.datetime.fromtimestamp(time.mktime(date)))
    if isinstance(date, datetime.datetime):
        date = date.date()
    return date


def copy_format_from_start(func):
    """
    作为装饰器从start(第一个参数)复制日期时间格式
    """
    @wraps(func)
    def copyer(*args, **kwargs):
        start = None if len(args) == 0 else args[0]
        res = func(*args, **kwargs)
        if start is not None and kwargs.get('copy_format', True):
            from dramkit.dttools import copy_format
            res = copy_format(res, start)
        return res
    return copyer



# -*- coding: utf-8 -*-

from __future__ import absolute_import, unicode_literals

__version__ = '3.0.2025'

from .constants.holiday import Holiday, holidays, workdays
from .holiday import get_dates
from .holiday import is_workday, is_holiday
from .holiday import get_holidays, get_workdays, find_workday
from .holiday import get_next_nth_workday, get_recent_workday, get_work_dates
from .holiday import get_holiday_detail, get_holiday_info_source
from .solar_terms import SolarTerms, get_solar_terms
from .xingzuo import XINGZUO, get_xingzuo
from .festival import FESTIVAL, FESTIVAL_LUNAR
from .lunar import sol2lun, lun2sol, sol2gz, hour2dz
from .lunar import get_tgdz_year, get_tgdz_date, get_tgdz_hour
from .bazi import get_bazi, get_bazi_lunar
from .bazi import get_wuxing, get_wuxing_lunar
from .bazi import get_marry_match, get_marry_match_lunar
from .bazi import get_chengming, get_chengming_lunar
from .bazi import get_age_by_shuxiang
from .trade_dates import is_tradeday, get_recent_tradeday
from .trade_dates import get_next_nth_tradeday, get_trade_dates


# TODO
# 沪深A股日历接入tushare
# 交易日历支持自定义数据
# 判断农历月大月小，判断是否闰月等
# 增加八字排盘算命等



# -*- coding: utf-8 -*-

from enum import Enum
import datetime
try:
    from .utils import trans_date
except:
    from chncal.utils import trans_date


class SolarTerms(Enum):
    """中国24节气"""

    the_beginning_of_spring = 'the Beginning of Spring', '立春'
    rain_water = 'Rain Water', '雨水'
    the_waking_of_insects = 'the Waking of Insects', '惊蛰'
    the_spring_equinox = 'the Spring Equinox', '春分'
    pure_brightness = 'Pure Brightness', '清明'
    grain_rain = 'Grain Rain', '谷雨'
    the_beginning_of_summer = 'the Beginning of Summer', '立夏'
    lesser_fullness_of_grain = 'Lesser Fullness of Grain', '小满'
    grain_in_beard = 'Grain in Beard', '芒种'
    the_summer_solstice = 'the Summer Solstice', '夏至'
    lesser_heat = 'Lesser Heat', '小暑'
    greater_heat = 'Greater Heat', '大暑'
    the_beginning_of_autumn = 'the Beginning of Autumn', '立秋'
    the_end_of_heat = 'the End of Heat', '处暑'
    white_dew = 'White Dew', '白露'
    the_autumn_equinox = 'the Autumn Equinox', '秋分'
    code_dew = 'Cold Dew', '寒露'
    frost_descent = "Frost's Descent", '霜降'
    the_beginning_of_winter = 'the Beginning of Winter', '立冬'
    lesser_snow = 'Lesser Snow', '小雪'
    greater_snow = 'Greater Snow', '大雪'
    the_winter_solstice = 'the Winter Solstice', '冬至'
    lesser_cold = 'Lesser Cold', '小寒'
    greater_cold = 'Greater Cold', '大寒'


# 月份和节气对应关系
SOLAR_TERMS_MONTH = {
    1: [SolarTerms.lesser_cold, SolarTerms.greater_cold],
    2: [SolarTerms.the_beginning_of_spring, SolarTerms.rain_water],
    3: [SolarTerms.the_waking_of_insects, SolarTerms.the_spring_equinox],
    4: [SolarTerms.pure_brightness, SolarTerms.grain_rain],
    5: [SolarTerms.the_beginning_of_summer, SolarTerms.lesser_fullness_of_grain],
    6: [SolarTerms.grain_in_beard, SolarTerms.the_summer_solstice],
    7: [SolarTerms.lesser_heat, SolarTerms.greater_heat],
    8: [SolarTerms.the_beginning_of_autumn, SolarTerms.the_end_of_heat],
    9: [SolarTerms.white_dew, SolarTerms.the_autumn_equinox],
    10: [SolarTerms.code_dew, SolarTerms.frost_descent],
    11: [SolarTerms.the_beginning_of_winter, SolarTerms.lesser_snow],
    12: [SolarTerms.greater_snow, SolarTerms.the_winter_solstice],
}


# 参考：https://www.jianshu.com/p/1f814c6bb475，有纠错
# 计算节气用的C值
# 2000年的小寒、大寒、立春、雨水按照20世纪的C值来算
SOLAR_TERMS_C_NUMS = {
    # 节气: [20世纪值， 21世纪值]
    SolarTerms.the_beginning_of_spring: [4.6295, 3.87],
    SolarTerms.rain_water: [19.4599, 18.73],
    SolarTerms.the_waking_of_insects: [6.3926, 5.63],
    SolarTerms.the_spring_equinox: [21.4155, 20.646],
    SolarTerms.pure_brightness: [5.59, 4.81],
    SolarTerms.grain_rain: [20.888, 20.1],
    SolarTerms.the_beginning_of_summer: [6.318, 5.52],
    SolarTerms.lesser_fullness_of_grain: [21.86, 21.04],
    SolarTerms.grain_in_beard: [6.5, 5.678],
    SolarTerms.the_summer_solstice: [22.2, 21.37],
    SolarTerms.lesser_heat: [7.928, 7.108],
    SolarTerms.greater_heat: [23.65, 22.83],
    SolarTerms.the_beginning_of_autumn: [8.35, 7.5],
    SolarTerms.the_end_of_heat: [23.95, 23.13],
    SolarTerms.white_dew: [8.44, 7.646],
    SolarTerms.the_autumn_equinox: [23.822, 23.042],
    SolarTerms.code_dew: [9.098, 8.318],
    SolarTerms.frost_descent: [24.218, 23.438],
    SolarTerms.the_beginning_of_winter: [8.218, 7.438],
    SolarTerms.lesser_snow: [23.08, 22.36],
    SolarTerms.greater_snow: [7.9, 7.18],
    SolarTerms.the_winter_solstice: [22.6, 21.94],
    SolarTerms.lesser_cold: [6.11, 5.4055],
    SolarTerms.greater_cold: [20.84, 20.12],
}

# 有些节气使用公式计算计算的不够准确，需要进行偏移
SOLAR_TERMS_DELTA = {
    (1902, SolarTerms.grain_in_beard): 1,
    (1911, SolarTerms.the_beginning_of_summer): 1,
    (1914, SolarTerms.the_beginning_of_spring): -1,
    (1914, SolarTerms.the_winter_solstice): 1,
    (1915, SolarTerms.the_waking_of_insects): -1,
    (1922, SolarTerms.greater_heat): 1,
    (1923, SolarTerms.rain_water): -1,
    (1925, SolarTerms.lesser_heat): 1,
    (1927, SolarTerms.white_dew): 1,
    (1928, SolarTerms.the_summer_solstice): 1,
    (1942, SolarTerms.the_autumn_equinox): 1,
    (1947, SolarTerms.the_beginning_of_spring): -1,
    (1947, SolarTerms.the_winter_solstice): 1,
    (1948, SolarTerms.the_waking_of_insects): -1,
    (1951, SolarTerms.the_winter_solstice): 1,
    (1978, SolarTerms.lesser_snow): 1,
    (1980, SolarTerms.the_winter_solstice): 1,
    (1982, SolarTerms.lesser_cold): 1,
    (1984, SolarTerms.the_winter_solstice): 1,
    (2000, SolarTerms.greater_cold): 1,
    (2002, SolarTerms.the_beginning_of_autumn): 1,
    (2008, SolarTerms.lesser_fullness_of_grain): 1,
    (2016, SolarTerms.lesser_heat): 1,
    (2019, SolarTerms.lesser_cold): -1,
    (2021, SolarTerms.the_winter_solstice): -1,
    (2026, SolarTerms.rain_water): -1,
    (2082, SolarTerms.greater_cold): 1,
    (2089, SolarTerms.frost_descent): 1,
    (2089, SolarTerms.the_beginning_of_winter): 1,
}


def get_solar_terms(start=None, end=None):
    """
    生成start和end日期之间的24节气，start或end不传，则取当前日期
    
    通用寿星公式：https://www.jianshu.com/p/1f814c6bb475，有纠错
    通式寿星公式：[Y×D+C]-L
    []里面取整数；Y=年数的后2位数；D=0.2422；L=Y/4，小寒、大寒、立春、雨水的 L=(Y-1)/4
    """
    start_bk = start
    start = trans_date(start)
    end = trans_date(end)
    if not 1900 <= start.year <= 2100 or not 1900 <= end.year <= 2100:
        raise NotImplementedError('only year between [1900, 2100] supported')
    D = 0.2422
    result = []
    year, month = start.year, start.month
    while year < end.year or (year == end.year and month <= end.month):
        # 按月计算节气
        for solar_term in SOLAR_TERMS_MONTH[month]:
            nums = SOLAR_TERMS_C_NUMS[solar_term]
            C = nums[0] if year < 2000 else nums[1]
            # 2000 年的小寒、大寒、立春、雨水按照 20 世纪的 C 值来算
            if year == 2000 and solar_term in [
                SolarTerms.lesser_cold,
                SolarTerms.greater_cold,
                SolarTerms.the_beginning_of_spring,
                SolarTerms.rain_water,
            ]:
                C = nums[0]
            Y = year % 100
            L = int(Y / 4)
            if solar_term in [
                SolarTerms.lesser_cold,
                SolarTerms.greater_cold,
                SolarTerms.the_beginning_of_spring,
                SolarTerms.rain_water,
            ]:
                L = int((Y - 1) / 4)
            day = int(Y * D + C) - L
            # 计算偏移量
            delta = SOLAR_TERMS_DELTA.get((year, solar_term))
            if delta:
                day += delta
            _date = datetime.date(year, month, day)
            if _date < start or _date > end:
                continue
            result.append((_date, solar_term.value[1]))
        if month == 12:
            year, month = year + 1, 1
        else:
            month += 1
    if start_bk is not None:
        from dramkit.dttools import copy_format
        result = [(copy_format(x[0], start_bk), x[1]) for x in result]
    return result



# * coding: utf8 *

import datetime
import pandas as pd
try:
    from .lunar import sol2gz, get_tgdz_hour, lun2sol, sol2lun, hour2dz, SX
    from .wuxing import TGWX, DZWX, TGDZNYWX
    from .marry_match import MARRY_MATCH
    from .constants.chengming import w_year, w_month, w_date, w_hour, song
except:
    from chncal.lunar import sol2gz, get_tgdz_hour, lun2sol, sol2lun, hour2dz, SX
    from chncal.wuxing import TGWX, DZWX, TGDZNYWX
    from chncal.marry_match import MARRY_MATCH
    from chncal.constants.chengming import w_year, w_month, w_date, w_hour, song
    
    
def get_bazi(time=None):
    """
    根据公历时间生成八字
    
    Examples
    --------
    >>> get_bazi('1992.05.14 18:00')
    """
    return sol2gz(time) + ',' + get_tgdz_hour(time) + '时'


def get_bazi_lunar(time, run=False):
    """
    根据农历时间生成八字，time格式必须形如'2023.02.30 19:30:20'，时分秒可以不写，
    run为True表示闰月日期
    
    Examples
    --------
    >>> get_bazi_lunar('2023.02.30')
    """
    assert isinstance(time, str) and '.' in time
    date = lun2sol(time[:10], run=run)
    time = date + time[10:]
    return get_bazi(time)


def get_wuxing(time=None):
    """根据公历时间获取五行信息"""
    bazi = get_bazi(time)
    wx_detail = {x: TGDZNYWX[x[:2]]+'(%s, %s)'%(TGWX[x[0]], DZWX[x[1]]) for x in bazi.split(',')}
    wx = [v[2] for k, v in wx_detail.items()]
    return wx, wx_detail


def get_wuxing_lunar(time, run=False):
    """根据农历时间获取五行信息，time格式必须形如'2023.02.30 19:30:20'，run为True表示闰月日期"""
    bazi = get_bazi_lunar(time, run=run)
    wx_detail = {x: TGDZNYWX[x[:2]]+'(%s, %s)'%(TGWX[x[0]], DZWX[x[1]]) for x in bazi.split(',')}
    wx = [v[2] for k, v in wx_detail.items()]
    return wx, wx_detail


def get_marry_match(time=None):
    """根据公历时间获取属相合婚信息"""
    sx = sol2gz(time)[3]
    return {sx: MARRY_MATCH[sol2gz(time)[3]]}


def get_marry_match_lunar(time, run=False):
    """根据农历时间获取属相合婚信息，time格式必须形如'2023.02.30 19:30:20'，run为True表示闰月日期"""
    assert isinstance(time, str) and '.' in time
    date = lun2sol(time[:10], run=run)
    sx = sol2gz(date)[3]
    return {sx: MARRY_MATCH[sol2gz(time)[3]]}


def _time2dz(time=None):
    if pd.isnull(time):
        hour = datetime.datetime.now().hour
    else:
        hour = pd.to_datetime(str(time)).hour
    return hour2dz(hour)


def get_chengming(time=None):
    """称命，传入公历时间"""
    bazi = get_bazi(time)
    date = sol2lun(time)
    wy = float(w_year[bazi[:5]])
    wm = float(w_month[date[5:7]])
    wd = float(w_date[date[8:]])
    wh = float(w_hour[_time2dz(time)])
    w = float(round(wy+wm+wd+wh, 2))
    sing = song[str(w)]
    result = {
        'weight': w,
        'bazi': bazi,
        'song': sing,
        'weight_split': (wy, wm, wd, wh)
    }
    return result


def get_chengming_lunar(time, run=False):
    """
    称命，传入农历时间
    
    Examples
    --------
    >>> get_chengming_lunar('2023.02.30 09:30:00')
    """
    assert isinstance(time, str) and '.' in time
    bazi = get_bazi_lunar(time, run=run)
    wy = float(w_year[bazi[:5]])
    wm = float(w_month[time[5:7]])
    wd = float(w_date[time[8:10]])
    hour = int(time[11:13]) if len(time) >= 13 else 0
    wh = float(w_hour[hour2dz(hour)])
    w = float(round(wy+wm+wd+wh, 2))
    sing = song[str(w)]
    result = {
        'weight': w,
        'bazi': bazi,
        'song': sing,
        'weight_split': (wy, wm, wd, wh)
    }
    return result


def get_age_by_shuxiang(shuxiang, year: int = None, return_n: int = 10):
    """根据属性获取可能年龄"""
    assert isinstance(shuxiang, str) and shuxiang in SX
    now_year = datetime.datetime.now().year if year is None else int(year)
    base_year, base_sx = 2022, '虎'
    res = {}
    n = 0
    year, idx = base_year, SX.index(base_sx)
    while n < return_n:
        if SX[idx] == shuxiang:
            age = now_year - year
            if age >= 0:
                res[year] = age
                n += 1
        year -= 1
        if idx == 0:
            idx = 11
        else:
            idx -= 1    
    return res



# -*- coding: utf-8 -*-

# this file is generated by:
# chncal.constant_creator.holiday.create_holiday_constant

from __future__ import absolute_import, unicode_literals

import datetime
from enum import Enum


class Holiday(Enum):
    
    def __new__(cls, english, chinese, days):        
        obj = object.__new__(cls)
        obj._value_ = english

        obj.chinese = chinese
        obj.days = days
        
        return obj

    # 常规假期天数
    new_years_day = "New Year's Day", '元旦', 1
    spring_festival = 'Spring Festival', '春节', 3
    tomb_sweeping_day = 'Tomb-sweeping Day', '清明', 1
    labour_day = 'Labour Day', '劳动节', 1
    dragon_boat_festival = 'Dragon Boat Festival', '端午', 1
    national_day = 'National Day', '国庆节', 3
    mid_autumn_festival = 'Mid-autumn Festival', '中秋', 1

    # special holidays
    anti_fascist_70th_day = 'Anti-Fascist 70th Day', '中国人民抗日战争暨世界反法西斯战争胜利70周年纪念日', 1


holidays = {
    datetime.date(year=2001, month=1, day=1): Holiday.new_years_day.value,
    datetime.date(year=2001, month=1, day=24): Holiday.spring_festival.value,
    datetime.date(year=2001, month=1, day=25): Holiday.spring_festival.value,
    datetime.date(year=2001, month=1, day=26): Holiday.spring_festival.value,
    datetime.date(year=2001, month=1, day=27): Holiday.spring_festival.value,
    datetime.date(year=2001, month=1, day=28): Holiday.spring_festival.value,
    datetime.date(year=2001, month=1, day=29): Holiday.spring_festival.value,
    datetime.date(year=2001, month=1, day=30): Holiday.spring_festival.value,
    datetime.date(year=2001, month=5, day=1): Holiday.labour_day.value,
    datetime.date(year=2001, month=5, day=2): Holiday.labour_day.value,
    datetime.date(year=2001, month=5, day=3): Holiday.labour_day.value,
    datetime.date(year=2001, month=5, day=4): Holiday.labour_day.value,
    datetime.date(year=2001, month=5, day=5): Holiday.labour_day.value,
    datetime.date(year=2001, month=5, day=6): Holiday.labour_day.value,
    datetime.date(year=2001, month=5, day=7): Holiday.labour_day.value,
    datetime.date(year=2001, month=10, day=1): Holiday.national_day.value,
    datetime.date(year=2001, month=10, day=2): Holiday.national_day.value,
    datetime.date(year=2001, month=10, day=3): Holiday.national_day.value,
    datetime.date(year=2001, month=10, day=4): Holiday.national_day.value,
    datetime.date(year=2001, month=10, day=5): Holiday.national_day.value,
    datetime.date(year=2001, month=10, day=6): Holiday.national_day.value,
    datetime.date(year=2001, month=10, day=7): Holiday.national_day.value,
    datetime.date(year=2002, month=1, day=1): Holiday.new_years_day.value,
    datetime.date(year=2002, month=1, day=2): Holiday.new_years_day.value,
    datetime.date(year=2002, month=1, day=3): Holiday.new_years_day.value,
    datetime.date(year=2002, month=2, day=12): Holiday.spring_festival.value,
    datetime.date(year=2002, month=2, day=13): Holiday.spring_festival.value,
    datetime.date(year=2002, month=2, day=14): Holiday.spring_festival.value,
    datetime.date(year=2002, month=2, day=15): Holiday.spring_festival.value,
    datetime.date(year=2002, month=2, day=16): Holiday.spring_festival.value,
    datetime.date(year=2002, month=2, day=17): Holiday.spring_festival.value,
    datetime.date(year=2002, month=2, day=18): Holiday.spring_festival.value,
    datetime.date(year=2002, month=5, day=1): Holiday.labour_day.value,
    datetime.date(year=2002, month=5, day=2): Holiday.labour_day.value,
    datetime.date(year=2002, month=5, day=3): Holiday.labour_day.value,
    datetime.date(year=2002, month=5, day=4): Holiday.labour_day.value,
    datetime.date(year=2002, month=5, day=5): Holiday.labour_day.value,
    datetime.date(year=2002, month=5, day=6): Holiday.labour_day.value,
    datetime.date(year=2002, month=5, day=7): Holiday.labour_day.value,
    datetime.date(year=2002, month=10, day=1): Holiday.national_day.value,
    datetime.date(year=2002, month=10, day=2): Holiday.national_day.value,
    datetime.date(year=2002, month=10, day=3): Holiday.national_day.value,
    datetime.date(year=2002, month=10, day=4): Holiday.national_day.value,
    datetime.date(year=2002, month=10, day=5): Holiday.national_day.value,
    datetime.date(year=2002, month=10, day=6): Holiday.national_day.value,
    datetime.date(year=2002, month=10, day=7): Holiday.national_day.value,
    datetime.date(year=2003, month=1, day=1): Holiday.new_years_day.value,
    datetime.date(year=2003, month=2, day=1): Holiday.spring_festival.value,
    datetime.date(year=2003, month=2, day=2): Holiday.spring_festival.value,
    datetime.date(year=2003, month=2, day=3): Holiday.spring_festival.value,
    datetime.date(year=2003, month=2, day=4): Holiday.spring_festival.value,
    datetime.date(year=2003, month=2, day=5): Holiday.spring_festival.value,
    datetime.date(year=2003, month=2, day=6): Holiday.spring_festival.value,
    datetime.date(year=2003, month=2, day=7): Holiday.spring_festival.value,
    datetime.date(year=2003, month=5, day=1): Holiday.labour_day.value,
    datetime.date(year=2003, month=5, day=2): Holiday.labour_day.value,
    datetime.date(year=2003, month=5, day=3): Holiday.labour_day.value,
    datetime.date(year=2003, month=5, day=4): Holiday.labour_day.value,
    datetime.date(year=2003, month=5, day=5): Holiday.labour_day.value,
    datetime.date(year=2003, month=5, day=6): Holiday.labour_day.value,
    datetime.date(year=2003, month=5, day=7): Holiday.labour_day.value,
    datetime.date(year=2003, month=10, day=1): Holiday.national_day.value,
    datetime.date(year=2003, month=10, day=2): Holiday.national_day.value,
    datetime.date(year=2003, month=10, day=3): Holiday.national_day.value,
    datetime.date(year=2003, month=10, day=4): Holiday.national_day.value,
    datetime.date(year=2003, month=10, day=5): Holiday.national_day.value,
    datetime.date(year=2003, month=10, day=6): Holiday.national_day.value,
    datetime.date(year=2003, month=10, day=7): Holiday.national_day.value,
    datetime.date(year=2004, month=1, day=1): Holiday.new_years_day.value,
    datetime.date(year=2004, month=1, day=22): Holiday.spring_festival.value,
    datetime.date(year=2004, month=1, day=23): Holiday.spring_festival.value,
    datetime.date(year=2004, month=1, day=24): Holiday.spring_festival.value,
    datetime.date(year=2004, month=1, day=25): Holiday.spring_festival.value,
    datetime.date(year=2004, month=1, day=26): Holiday.spring_festival.value,
    datetime.date(year=2004, month=1, day=27): Holiday.spring_festival.value,
    datetime.date(year=2004, month=1, day=28): Holiday.spring_festival.value,
    datetime.date(year=2004, month=5, day=1): Holiday.labour_day.value,
    datetime.date(year=2004, month=5, day=2): Holiday.labour_day.value,
    datetime.date(year=2004, month=5, day=3): Holiday.labour_day.value,
    datetime.date(year=2004, month=5, day=4): Holiday.labour_day.value,
    datetime.date(year=2004, month=5, day=5): Holiday.labour_day.value,
    datetime.date(year=2004, month=5, day=6): Holiday.labour_day.value,
    datetime.date(year=2004, month=5, day=7): Holiday.labour_day.value,
    datetime.date(year=2004, month=10, day=1): Holiday.national_day.value,
    datetime.date(year=2004, month=10, day=2): Holiday.national_day.value,
    datetime.date(year=2004, month=10, day=3): Holiday.national_day.value,
    datetime.date(year=2004, month=10, day=4): Holiday.national_day.value,
    datetime.date(year=2004, month=10, day=5): Holiday.national_day.value,
    datetime.date(year=2004, month=10, day=6): Holiday.national_day.value,
    datetime.date(year=2004, month=10, day=7): Holiday.national_day.value,
    datetime.date(year=2005, month=1, day=1): Holiday.new_years_day.value,
    datetime.date(year=2005, month=1, day=2): Holiday.new_years_day.value,
    datetime.date(year=2005, month=1, day=3): Holiday.new_years_day.value,
    datetime.date(year=2005, month=2, day=9): Holiday.spring_festival.value,
    datetime.date(year=2005, month=2, day=10): Holiday.spring_festival.value,
    datetime.date(year=2005, month=2, day=11): Holiday.spring_festival.value,
    datetime.date(year=2005, month=2, day=12): Holiday.spring_festival.value,
    datetime.date(year=2005, month=2, day=13): Holiday.spring_festival.value,
    datetime.date(year=2005, month=2, day=14): Holiday.spring_festival.value,
    datetime.date(year=2005, month=2, day=15): Holiday.spring_festival.value,
    datetime.date(year=2005, month=5, day=1): Holiday.labour_day.value,
    datetime.date(year=2005, month=5, day=2): Holiday.labour_day.value,
    datetime.date(year=2005, month=5, day=3): Holiday.labour_day.value,
    datetime.date(year=2005, month=5, day=4): Holiday.labour_day.value,
    datetime.date(year=2005, month=5, day=5): Holiday.labour_day.value,
    datetime.date(year=2005, month=5, day=6): Holiday.labour_day.value,
    datetime.date(year=2005, month=5, day=7): Holiday.labour_day.value,
    datetime.date(year=2005, month=10, day=1): Holiday.national_day.value,
    datetime.date(year=2005, month=10, day=2): Holiday.national_day.value,
    datetime.date(year=2005, month=10, day=3): Holiday.national_day.value,
    datetime.date(year=2005, month=10, day=4): Holiday.national_day.value,
    datetime.date(year=2005, month=10, day=5): Holiday.national_day.value,
    datetime.date(year=2005, month=10, day=6): Holiday.national_day.value,
    datetime.date(year=2005, month=10, day=7): Holiday.national_day.value,
    datetime.date(year=2006, month=1, day=1): Holiday.new_years_day.value,
    datetime.date(year=2006, month=1, day=2): Holiday.new_years_day.value,
    datetime.date(year=2006, month=1, day=3): Holiday.new_years_day.value,
    datetime.date(year=2006, month=1, day=29): Holiday.spring_festival.value,
    datetime.date(year=2006, month=1, day=30): Holiday.spring_festival.value,
    datetime.date(year=2006, month=1, day=31): Holiday.spring_festival.value,
    datetime.date(year=2006, month=2, day=1): Holiday.spring_festival.value,
    datetime.date(year=2006, month=2, day=2): Holiday.spring_festival.value,
    datetime.date(year=2006, month=2, day=3): Holiday.spring_festival.value,
    datetime.date(year=2006, month=2, day=4): Holiday.spring_festival.value,
    datetime.date(year=2006, month=5, day=1): Holiday.labour_day.value,
    datetime.date(year=2006, month=5, day=2): Holiday.labour_day.value,
    datetime.date(year=2006, month=5, day=3): Holiday.labour_day.value,
    datetime.date(year=2006, month=5, day=4): Holiday.labour_day.value,
    datetime.date(year=2006, month=5, day=5): Holiday.labour_day.value,
    datetime.date(year=2006, month=5, day=6): Holiday.labour_day.value,
    datetime.date(year=2006, month=5, day=7): Holiday.labour_day.value,
    datetime.date(year=2006, month=10, day=1): Holiday.national_day.value,
    datetime.date(year=2006, month=10, day=2): Holiday.national_day.value,
    datetime.date(year=2006, month=10, day=3): Holiday.national_day.value,
    datetime.date(year=2006, month=10, day=4): Holiday.national_day.value,
    datetime.date(year=2006, month=10, day=5): Holiday.national_day.value,
    datetime.date(year=2006, month=10, day=6): Holiday.national_day.value,
    datetime.date(year=2006, month=10, day=7): Holiday.national_day.value,
    datetime.date(year=2007, month=1, day=1): Holiday.new_years_day.value,
    datetime.date(year=2007, month=1, day=2): Holiday.new_years_day.value,
    datetime.date(year=2007, month=1, day=3): Holiday.new_years_day.value,
    datetime.date(year=2007, month=2, day=18): Holiday.spring_festival.value,
    datetime.date(year=2007, month=2, day=19): Holiday.spring_festival.value,
    datetime.date(year=2007, month=2, day=20): Holiday.spring_festival.value,
    datetime.date(year=2007, month=2, day=21): Holiday.spring_festival.value,
    datetime.date(year=2007, month=2, day=22): Holiday.spring_festival.value,
    datetime.date(year=2007, month=2, day=23): Holiday.spring_festival.value,
    datetime.date(year=2007, month=2, day=24): Holiday.spring_festival.value,
    datetime.date(year=2007, month=5, day=1): Holiday.labour_day.value,
    datetime.date(year=2007, month=5, day=7): Holiday.labour_day.value,
    datetime.date(year=2007, month=10, day=1): Holiday.national_day.value,
    datetime.date(year=2007, month=10, day=2): Holiday.national_day.value,
    datetime.date(year=2007, month=10, day=3): Holiday.national_day.value,
    datetime.date(year=2007, month=10, day=4): Holiday.national_day.value,
    datetime.date(year=2007, month=10, day=5): Holiday.national_day.value,
    datetime.date(year=2007, month=10, day=6): Holiday.national_day.value,
    datetime.date(year=2007, month=10, day=7): Holiday.national_day.value,
    datetime.date(year=2007, month=12, day=30): Holiday.new_years_day.value,
    datetime.date(year=2007, month=12, day=31): Holiday.new_years_day.value,
    datetime.date(year=2008, month=1, day=1): Holiday.new_years_day.value,
    datetime.date(year=2008, month=2, day=6): Holiday.spring_festival.value,
    datetime.date(year=2008, month=2, day=7): Holiday.spring_festival.value,
    datetime.date(year=2008, month=2, day=8): Holiday.spring_festival.value,
    datetime.date(year=2008, month=2, day=9): Holiday.spring_festival.value,
    datetime.date(year=2008, month=2, day=10): Holiday.spring_festival.value,
    datetime.date(year=2008, month=2, day=11): Holiday.spring_festival.value,
    datetime.date(year=2008, month=2, day=12): Holiday.spring_festival.value,
    datetime.date(year=2008, month=4, day=4): Holiday.tomb_sweeping_day.value,
    datetime.date(year=2008, month=4, day=5): Holiday.tomb_sweeping_day.value,
    datetime.date(year=2008, month=4, day=6): Holiday.tomb_sweeping_day.value,
    datetime.date(year=2008, month=5, day=1): Holiday.labour_day.value,
    datetime.date(year=2008, month=5, day=2): Holiday.labour_day.value,
    datetime.date(year=2008, month=5, day=3): Holiday.labour_day.value,
    datetime.date(year=2008, month=6, day=7): Holiday.dragon_boat_festival.value,
    datetime.date(year=2008, month=6, day=8): Holiday.dragon_boat_festival.value,
    datetime.date(year=2008, month=6, day=9): Holiday.dragon_boat_festival.value,
    datetime.date(year=2008, month=9, day=13): Holiday.mid_autumn_festival.value,
    datetime.date(year=2008, month=9, day=14): Holiday.mid_autumn_festival.value,
    datetime.date(year=2008, month=9, day=15): Holiday.mid_autumn_festival.value,
    datetime.date(year=2008, month=9, day=29): Holiday.national_day.value,
    datetime.date(year=2008, month=9, day=30): Holiday.national_day.value,
    datetime.date(year=2008, month=10, day=1): Holiday.national_day.value,
    datetime.date(year=2008, month=10, day=2): Holiday.national_day.value,
    datetime.date(year=2008, month=10, day=3): Holiday.national_day.value,
    datetime.date(year=2008, month=10, day=4): Holiday.national_day.value,
    datetime.date(year=2008, month=10, day=5): Holiday.national_day.value,
    datetime.date(year=2009, month=1, day=1): Holiday.new_years_day.value,
    datetime.date(year=2009, month=1, day=2): Holiday.new_years_day.value,
    datetime.date(year=2009, month=1, day=3): Holiday.new_years_day.value,
    datetime.date(year=2009, month=1, day=25): Holiday.spring_festival.value,
    datetime.date(year=2009, month=1, day=26): Holiday.spring_festival.value,
    datetime.date(year=2009, month=1, day=27): Holiday.spring_festival.value,
    datetime.date(year=2009, month=1, day=28): Holiday.spring_festival.value,
    datetime.date(year=2009, month=1, day=29): Holiday.spring_festival.value,
    datetime.date(year=2009, month=1, day=30): Holiday.spring_festival.value,
    datetime.date(year=2009, month=1, day=31): Holiday.spring_festival.value,
    datetime.date(year=2009, month=4, day=4): Holiday.tomb_sweeping_day.value,
    datetime.date(year=2009, month=4, day=5): Holiday.tomb_sweeping_day.value,
    datetime.date(year=2009, month=4, day=6): Holiday.tomb_sweeping_day.value,
    datetime.date(year=2009, month=5, day=1): Holiday.labour_day.value,
    datetime.date(year=2009, month=5, day=2): Holiday.labour_day.value,
    datetime.date(year=2009, month=5, day=3): Holiday.labour_day.value,
    datetime.date(year=2009, month=5, day=28): Holiday.dragon_boat_festival.value,
    datetime.date(year=2009, month=5, day=29): Holiday.dragon_boat_festival.value,
    datetime.date(year=2009, month=5, day=30): Holiday.dragon_boat_festival.value,
    datetime.date(year=2009, month=10, day=1): Holiday.national_day.value,
    datetime.date(year=2009, month=10, day=2): Holiday.national_day.value,
    datetime.date(year=2009, month=10, day=3): Holiday.mid_autumn_festival.value,
    datetime.date(year=2009, month=10, day=4): Holiday.national_day.value,
    datetime.date(year=2009, month=10, day=5): Holiday.national_day.value,
    datetime.date(year=2009, month=10, day=6): Holiday.national_day.value,
    datetime.date(year=2009, month=10, day=7): Holiday.national_day.value,
    datetime.date(year=2009, month=10, day=8): Holiday.national_day.value,
    datetime.date(year=2010, month=1, day=1): Holiday.new_years_day.value,
    datetime.date(year=2010, month=1, day=2): Holiday.new_years_day.value,
    datetime.date(year=2010, month=1, day=3): Holiday.new_years_day.value,
    datetime.date(year=2010, month=2, day=13): Holiday.spring_festival.value,
    datetime.date(year=2010, month=2, day=14): Holiday.spring_festival.value,
    datetime.date(year=2010, month=2, day=15): Holiday.spring_festival.value,
    datetime.date(year=2010, month=2, day=16): Holiday.spring_festival.value,
    datetime.date(year=2010, month=2, day=17): Holiday.spring_festival.value,
    datetime.date(year=2010, month=2, day=18): Holiday.spring_festival.value,
    datetime.date(year=2010, month=2, day=19): Holiday.spring_festival.value,
    datetime.date(year=2010, month=4, day=3): Holiday.tomb_sweeping_day.value,
    datetime.date(year=2010, month=4, day=4): Holiday.tomb_sweeping_day.value,
    datetime.date(year=2010, month=4, day=5): Holiday.tomb_sweeping_day.value,
    datetime.date(year=2010, month=5, day=1): Holiday.labour_day.value,
    datetime.date(year=2010, month=5, day=2): Holiday.labour_day.value,
    datetime.date(year=2010, month=5, day=3): Holiday.labour_day.value,
    datetime.date(year=2010, month=6, day=14): Holiday.dragon_boat_festival.value,
    datetime.date(year=2010, month=6, day=15): Holiday.dragon_boat_festival.value,
    datetime.date(year=2010, month=6, day=16): Holiday.dragon_boat_festival.value,
    datetime.date(year=2010, month=9, day=22): Holiday.mid_autumn_festival.value,
    datetime.date(year=2010, month=9, day=24): Holiday.mid_autumn_festival.value,
    datetime.date(year=2010, month=10, day=1): Holiday.national_day.value,
    datetime.date(year=2010, month=10, day=2): Holiday.national_day.value,
    datetime.date(year=2010, month=10, day=3): Holiday.national_day.value,
    datetime.date(year=2010, month=10, day=4): Holiday.national_day.value,
    datetime.date(year=2010, month=10, day=5): Holiday.national_day.value,
    datetime.date(year=2010, month=10, day=6): Holiday.national_day.value,
    datetime.date(year=2010, month=10, day=7): Holiday.national_day.value,
    datetime.date(year=2011, month=1, day=1): Holiday.new_years_day.value,
    datetime.date(year=2011, month=1, day=2): Holiday.new_years_day.value,
    datetime.date(year=2011, month=1, day=3): Holiday.new_years_day.value,
    datetime.date(year=2011, month=2, day=2): Holiday.spring_festival.value,
    datetime.date(year=2011, month=2, day=3): Holiday.spring_festival.value,
    datetime.date(year=2011, month=2, day=4): Holiday.spring_festival.value,
    datetime.date(year=2011, month=2, day=5): Holiday.spring_festival.value,
    datetime.date(year=2011, month=2, day=6): Holiday.spring_festival.value,
    datetime.date(year=2011, month=2, day=7): Holiday.spring_festival.value,
    datetime.date(year=2011, month=2, day=8): Holiday.spring_festival.value,
    datetime.date(year=2011, month=4, day=3): Holiday.tomb_sweeping_day.value,
    datetime.date(year=2011, month=4, day=4): Holiday.tomb_sweeping_day.value,
    datetime.date(year=2011, month=4, day=5): Holiday.tomb_sweeping_day.value,
    datetime.date(year=2011, month=4, day=30): Holiday.labour_day.value,
    datetime.date(year=2011, month=5, day=1): Holiday.labour_day.value,
    datetime.date(year=2011, month=5, day=2): Holiday.labour_day.value,
    datetime.date(year=2011, month=6, day=4): Holiday.dragon_boat_festival.value,
    datetime.date(year=2011, month=6, day=5): Holiday.dragon_boat_festival.value,
    datetime.date(year=2011, month=6, day=6): Holiday.dragon_boat_festival.value,
    datetime.date(year=2011, month=9, day=10): Holiday.mid_autumn_festival.value,
    datetime.date(year=2011, month=9, day=11): Holiday.mid_autumn_festival.value,
    datetime.date(year=2011, month=9, day=12): Holiday.mid_autumn_festival.value,
    datetime.date(year=2011, month=10, day=1): Holiday.national_day.value,
    datetime.date(year=2011, month=10, day=2): Holiday.national_day.value,
    datetime.date(year=2011, month=10, day=3): Holiday.national_day.value,
    datetime.date(year=2011, month=10, day=4): Holiday.national_day.value,
    datetime.date(year=2011, month=10, day=5): Holiday.national_day.value,
    datetime.date(year=2011, month=10, day=6): Holiday.national_day.value,
    datetime.date(year=2011, month=10, day=7): Holiday.national_day.value,
    datetime.date(year=2012, month=1, day=1): Holiday.new_years_day.value,
    datetime.date(year=2012, month=1, day=2): Holiday.new_years_day.value,
    datetime.date(year=2012, month=1, day=3): Holiday.new_years_day.value,
    datetime.date(year=2012, month=1, day=22): Holiday.spring_festival.value,
    datetime.date(year=2012, month=1, day=23): Holiday.spring_festival.value,
    datetime.date(year=2012, month=1, day=24): Holiday.spring_festival.value,
    datetime.date(year=2012, month=1, day=25): Holiday.spring_festival.value,
    datetime.date(year=2012, month=1, day=26): Holiday.spring_festival.value,
    datetime.date(year=2012, month=1, day=27): Holiday.spring_festival.value,
    datetime.date(year=2012, month=1, day=28): Holiday.spring_festival.value,
    datetime.date(year=2012, month=4, day=2): Holiday.tomb_sweeping_day.value,
    datetime.date(year=2012, month=4, day=3): Holiday.tomb_sweeping_day.value,
    datetime.date(year=2012, month=4, day=4): Holiday.tomb_sweeping_day.value,
    datetime.date(year=2012, month=4, day=29): Holiday.labour_day.value,
    datetime.date(year=2012, month=4, day=30): Holiday.labour_day.value,
    datetime.date(year=2012, month=5, day=1): Holiday.labour_day.value,
    datetime.date(year=2012, month=6, day=22): Holiday.dragon_boat_festival.value,
    datetime.date(year=2012, month=6, day=23): Holiday.dragon_boat_festival.value,
    datetime.date(year=2012, month=6, day=24): Holiday.dragon_boat_festival.value,
    datetime.date(year=2012, month=9, day=30): Holiday.mid_autumn_festival.value,
    datetime.date(year=2012, month=10, day=1): Holiday.national_day.value,
    datetime.date(year=2012, month=10, day=2): Holiday.national_day.value,
    datetime.date(year=2012, month=10, day=3): Holiday.national_day.value,
    datetime.date(year=2012, month=10, day=4): Holiday.national_day.value,
    datetime.date(year=2012, month=10, day=5): Holiday.national_day.value,
    datetime.date(year=2012, month=10, day=6): Holiday.national_day.value,
    datetime.date(year=2012, month=10, day=7): Holiday.national_day.value,
    datetime.date(year=2013, month=1, day=1): Holiday.new_years_day.value,
    datetime.date(year=2013, month=1, day=2): Holiday.new_years_day.value,
    datetime.date(year=2013, month=1, day=3): Holiday.new_years_day.value,
    datetime.date(year=2013, month=2, day=9): Holiday.spring_festival.value,
    datetime.date(year=2013, month=2, day=10): Holiday.spring_festival.value,
    datetime.date(year=2013, month=2, day=11): Holiday.spring_festival.value,
    datetime.date(year=2013, month=2, day=12): Holiday.spring_festival.value,
    datetime.date(year=2013, month=2, day=13): Holiday.spring_festival.value,
    datetime.date(year=2013, month=2, day=14): Holiday.spring_festival.value,
    datetime.date(year=2013, month=2, day=15): Holiday.spring_festival.value,
    datetime.date(year=2013, month=4, day=4): Holiday.tomb_sweeping_day.value,
    datetime.date(year=2013, month=4, day=6): Holiday.tomb_sweeping_day.value,
    datetime.date(year=2013, month=4, day=29): Holiday.labour_day.value,
    datetime.date(year=2013, month=4, day=30): Holiday.labour_day.value,
    datetime.date(year=2013, month=5, day=1): Holiday.labour_day.value,
    datetime.date(year=2013, month=6, day=10): Holiday.dragon_boat_festival.value,
    datetime.date(year=2013, month=6, day=11): Holiday.dragon_boat_festival.value,
    datetime.date(year=2013, month=6, day=12): Holiday.dragon_boat_festival.value,
    datetime.date(year=2013, month=9, day=19): Holiday.mid_autumn_festival.value,
    datetime.date(year=2013, month=9, day=21): Holiday.mid_autumn_festival.value,
    datetime.date(year=2013, month=10, day=1): Holiday.national_day.value,
    datetime.date(year=2013, month=10, day=2): Holiday.national_day.value,
    datetime.date(year=2013, month=10, day=3): Holiday.national_day.value,
    datetime.date(year=2013, month=10, day=4): Holiday.national_day.value,
    datetime.date(year=2013, month=10, day=5): Holiday.national_day.value,
    datetime.date(year=2013, month=10, day=6): Holiday.national_day.value,
    datetime.date(year=2013, month=10, day=7): Holiday.national_day.value,
    datetime.date(year=2014, month=1, day=1): Holiday.new_years_day.value,
    datetime.date(year=2014, month=1, day=31): Holiday.spring_festival.value,
    datetime.date(year=2014, month=2, day=1): Holiday.spring_festival.value,
    datetime.date(year=2014, month=2, day=2): Holiday.spring_festival.value,
    datetime.date(year=2014, month=2, day=3): Holiday.spring_festival.value,
    datetime.date(year=2014, month=2, day=4): Holiday.spring_festival.value,
    datetime.date(year=2014, month=2, day=5): Holiday.spring_festival.value,
    datetime.date(year=2014, month=2, day=6): Holiday.spring_festival.value,
    datetime.date(year=2014, month=4, day=5): Holiday.tomb_sweeping_day.value,
    datetime.date(year=2014, month=4, day=6): Holiday.tomb_sweeping_day.value,
    datetime.date(year=2014, month=4, day=7): Holiday.tomb_sweeping_day.value,
    datetime.date(year=2014, month=5, day=1): Holiday.labour_day.value,
    datetime.date(year=2014, month=5, day=2): Holiday.labour_day.value,
    datetime.date(year=2014, month=5, day=3): Holiday.labour_day.value,
    datetime.date(year=2014, month=6, day=2): Holiday.dragon_boat_festival.value,
    datetime.date(year=2014, month=9, day=8): Holiday.mid_autumn_festival.value,
    datetime.date(year=2014, month=10, day=1): Holiday.national_day.value,
    datetime.date(year=2014, month=10, day=2): Holiday.national_day.value,
    datetime.date(year=2014, month=10, day=3): Holiday.national_day.value,
    datetime.date(year=2014, month=10, day=4): Holiday.national_day.value,
    datetime.date(year=2014, month=10, day=5): Holiday.national_day.value,
    datetime.date(year=2014, month=10, day=6): Holiday.national_day.value,
    datetime.date(year=2014, month=10, day=7): Holiday.national_day.value,
    datetime.date(year=2015, month=1, day=1): Holiday.new_years_day.value,
    datetime.date(year=2015, month=1, day=2): Holiday.new_years_day.value,
    datetime.date(year=2015, month=1, day=3): Holiday.new_years_day.value,
    datetime.date(year=2015, month=2, day=18): Holiday.spring_festival.value,
    datetime.date(year=2015, month=2, day=19): Holiday.spring_festival.value,
    datetime.date(year=2015, month=2, day=20): Holiday.spring_festival.value,
    datetime.date(year=2015, month=2, day=21): Holiday.spring_festival.value,
    datetime.date(year=2015, month=2, day=22): Holiday.spring_festival.value,
    datetime.date(year=2015, month=2, day=23): Holiday.spring_festival.value,
    datetime.date(year=2015, month=2, day=24): Holiday.spring_festival.value,
    datetime.date(year=2015, month=4, day=5): Holiday.tomb_sweeping_day.value,
    datetime.date(year=2015, month=4, day=6): Holiday.tomb_sweeping_day.value,
    datetime.date(year=2015, month=5, day=1): Holiday.labour_day.value,
    datetime.date(year=2015, month=6, day=20): Holiday.dragon_boat_festival.value,
    datetime.date(year=2015, month=6, day=22): Holiday.dragon_boat_festival.value,
    datetime.date(year=2015, month=9, day=3): Holiday.anti_fascist_70th_day.value,
    datetime.date(year=2015, month=9, day=4): Holiday.anti_fascist_70th_day.value,
    datetime.date(year=2015, month=9, day=27): Holiday.mid_autumn_festival.value,
    datetime.date(year=2015, month=10, day=1): Holiday.national_day.value,
    datetime.date(year=2015, month=10, day=2): Holiday.national_day.value,
    datetime.date(year=2015, month=10, day=3): Holiday.national_day.value,
    datetime.date(year=2015, month=10, day=4): Holiday.national_day.value,
    datetime.date(year=2015, month=10, day=5): Holiday.national_day.value,
    datetime.date(year=2015, month=10, day=6): Holiday.national_day.value,
    datetime.date(year=2015, month=10, day=7): Holiday.national_day.value,
    datetime.date(year=2016, month=1, day=1): Holiday.new_years_day.value,
    datetime.date(year=2016, month=2, day=7): Holiday.spring_festival.value,
    datetime.date(year=2016, month=2, day=8): Holiday.spring_festival.value,
    datetime.date(year=2016, month=2, day=9): Holiday.spring_festival.value,
    datetime.date(year=2016, month=2, day=10): Holiday.spring_festival.value,
    datetime.date(year=2016, month=2, day=11): Holiday.spring_festival.value,
    datetime.date(year=2016, month=2, day=12): Holiday.spring_festival.value,
    datetime.date(year=2016, month=2, day=13): Holiday.spring_festival.value,
    datetime.date(year=2016, month=4, day=4): Holiday.tomb_sweeping_day.value,
    datetime.date(year=2016, month=5, day=1): Holiday.labour_day.value,
    datetime.date(year=2016, month=5, day=2): Holiday.labour_day.value,
    datetime.date(year=2016, month=6, day=9): Holiday.dragon_boat_festival.value,
    datetime.date(year=2016, month=6, day=10): Holiday.dragon_boat_festival.value,
    datetime.date(year=2016, month=6, day=11): Holiday.dragon_boat_festival.value,
    datetime.date(year=2016, month=9, day=15): Holiday.mid_autumn_festival.value,
    datetime.date(year=2016, month=9, day=16): Holiday.mid_autumn_festival.value,
    datetime.date(year=2016, month=9, day=17): Holiday.mid_autumn_festival.value,
    datetime.date(year=2016, month=10, day=1): Holiday.national_day.value,
    datetime.date(year=2016, month=10, day=2): Holiday.national_day.value,
    datetime.date(year=2016, month=10, day=3): Holiday.national_day.value,
    datetime.date(year=2016, month=10, day=4): Holiday.national_day.value,
    datetime.date(year=2016, month=10, day=5): Holiday.national_day.value,
    datetime.date(year=2016, month=10, day=6): Holiday.national_day.value,
    datetime.date(year=2016, month=10, day=7): Holiday.national_day.value,
    datetime.date(year=2017, month=1, day=1): Holiday.new_years_day.value,
    datetime.date(year=2017, month=1, day=2): Holiday.new_years_day.value,
    datetime.date(year=2017, month=1, day=27): Holiday.spring_festival.value,
    datetime.date(year=2017, month=1, day=28): Holiday.spring_festival.value,
    datetime.date(year=2017, month=1, day=29): Holiday.spring_festival.value,
    datetime.date(year=2017, month=1, day=30): Holiday.spring_festival.value,
    datetime.date(year=2017, month=1, day=31): Holiday.spring_festival.value,
    datetime.date(year=2017, month=2, day=1): Holiday.spring_festival.value,
    datetime.date(year=2017, month=2, day=2): Holiday.spring_festival.value,
    datetime.date(year=2017, month=4, day=2): Holiday.tomb_sweeping_day.value,
    datetime.date(year=2017, month=4, day=3): Holiday.tomb_sweeping_day.value,
    datetime.date(year=2017, month=4, day=4): Holiday.tomb_sweeping_day.value,
    datetime.date(year=2017, month=5, day=1): Holiday.labour_day.value,
    datetime.date(year=2017, month=5, day=28): Holiday.dragon_boat_festival.value,
    datetime.date(year=2017, month=5, day=29): Holiday.dragon_boat_festival.value,
    datetime.date(year=2017, month=5, day=30): Holiday.dragon_boat_festival.value,
    datetime.date(year=2017, month=10, day=1): Holiday.national_day.value,
    datetime.date(year=2017, month=10, day=2): Holiday.national_day.value,
    datetime.date(year=2017, month=10, day=3): Holiday.national_day.value,
    datetime.date(year=2017, month=10, day=4): Holiday.mid_autumn_festival.value,
    datetime.date(year=2017, month=10, day=5): Holiday.national_day.value,
    datetime.date(year=2017, month=10, day=6): Holiday.national_day.value,
    datetime.date(year=2017, month=10, day=7): Holiday.national_day.value,
    datetime.date(year=2017, month=10, day=8): Holiday.national_day.value,
    datetime.date(year=2018, month=1, day=1): Holiday.new_years_day.value,
    datetime.date(year=2018, month=2, day=15): Holiday.spring_festival.value,
    datetime.date(year=2018, month=2, day=16): Holiday.spring_festival.value,
    datetime.date(year=2018, month=2, day=17): Holiday.spring_festival.value,
    datetime.date(year=2018, month=2, day=18): Holiday.spring_festival.value,
    datetime.date(year=2018, month=2, day=19): Holiday.spring_festival.value,
    datetime.date(year=2018, month=2, day=20): Holiday.spring_festival.value,
    datetime.date(year=2018, month=2, day=21): Holiday.spring_festival.value,
    datetime.date(year=2018, month=4, day=5): Holiday.tomb_sweeping_day.value,
    datetime.date(year=2018, month=4, day=6): Holiday.tomb_sweeping_day.value,
    datetime.date(year=2018, month=4, day=7): Holiday.tomb_sweeping_day.value,
    datetime.date(year=2018, month=4, day=29): Holiday.labour_day.value,
    datetime.date(year=2018, month=4, day=30): Holiday.labour_day.value,
    datetime.date(year=2018, month=5, day=1): Holiday.labour_day.value,
    datetime.date(year=2018, month=6, day=18): Holiday.dragon_boat_festival.value,
    datetime.date(year=2018, month=9, day=24): Holiday.mid_autumn_festival.value,
    datetime.date(year=2018, month=10, day=1): Holiday.national_day.value,
    datetime.date(year=2018, month=10, day=2): Holiday.national_day.value,
    datetime.date(year=2018, month=10, day=3): Holiday.national_day.value,
    datetime.date(year=2018, month=10, day=4): Holiday.national_day.value,
    datetime.date(year=2018, month=10, day=5): Holiday.national_day.value,
    datetime.date(year=2018, month=10, day=6): Holiday.national_day.value,
    datetime.date(year=2018, month=10, day=7): Holiday.national_day.value,
    datetime.date(year=2018, month=12, day=30): Holiday.new_years_day.value,
    datetime.date(year=2018, month=12, day=31): Holiday.new_years_day.value,
    datetime.date(year=2019, month=1, day=1): Holiday.new_years_day.value,
    datetime.date(year=2019, month=2, day=4): Holiday.spring_festival.value,
    datetime.date(year=2019, month=2, day=5): Holiday.spring_festival.value,
    datetime.date(year=2019, month=2, day=6): Holiday.spring_festival.value,
    datetime.date(year=2019, month=2, day=7): Holiday.spring_festival.value,
    datetime.date(year=2019, month=2, day=8): Holiday.spring_festival.value,
    datetime.date(year=2019, month=2, day=9): Holiday.spring_festival.value,
    datetime.date(year=2019, month=2, day=10): Holiday.spring_festival.value,
    datetime.date(year=2019, month=4, day=5): Holiday.tomb_sweeping_day.value,
    datetime.date(year=2019, month=5, day=1): Holiday.labour_day.value,
    datetime.date(year=2019, month=5, day=2): Holiday.labour_day.value,
    datetime.date(year=2019, month=5, day=3): Holiday.labour_day.value,
    datetime.date(year=2019, month=5, day=4): Holiday.labour_day.value,
    datetime.date(year=2019, month=6, day=7): Holiday.dragon_boat_festival.value,
    datetime.date(year=2019, month=9, day=13): Holiday.mid_autumn_festival.value,
    datetime.date(year=2019, month=10, day=1): Holiday.national_day.value,
    datetime.date(year=2019, month=10, day=2): Holiday.national_day.value,
    datetime.date(year=2019, month=10, day=3): Holiday.national_day.value,
    datetime.date(year=2019, month=10, day=4): Holiday.national_day.value,
    datetime.date(year=2019, month=10, day=5): Holiday.national_day.value,
    datetime.date(year=2019, month=10, day=6): Holiday.national_day.value,
    datetime.date(year=2019, month=10, day=7): Holiday.national_day.value,
    datetime.date(year=2020, month=1, day=1): Holiday.new_years_day.value,
    datetime.date(year=2020, month=1, day=24): Holiday.spring_festival.value,
    datetime.date(year=2020, month=1, day=25): Holiday.spring_festival.value,
    datetime.date(year=2020, month=1, day=26): Holiday.spring_festival.value,
    datetime.date(year=2020, month=1, day=27): Holiday.spring_festival.value,
    datetime.date(year=2020, month=1, day=28): Holiday.spring_festival.value,
    datetime.date(year=2020, month=1, day=29): Holiday.spring_festival.value,
    datetime.date(year=2020, month=1, day=30): Holiday.spring_festival.value,
    datetime.date(year=2020, month=1, day=31): Holiday.spring_festival.value,
    datetime.date(year=2020, month=2, day=1): Holiday.spring_festival.value,
    datetime.date(year=2020, month=2, day=2): Holiday.spring_festival.value,
    datetime.date(year=2020, month=4, day=4): Holiday.tomb_sweeping_day.value,
    datetime.date(year=2020, month=4, day=5): Holiday.tomb_sweeping_day.value,
    datetime.date(year=2020, month=4, day=6): Holiday.tomb_sweeping_day.value,
    datetime.date(year=2020, month=5, day=1): Holiday.labour_day.value,
    datetime.date(year=2020, month=5, day=2): Holiday.labour_day.value,
    datetime.date(year=2020, month=5, day=3): Holiday.labour_day.value,
    datetime.date(year=2020, month=5, day=4): Holiday.labour_day.value,
    datetime.date(year=2020, month=5, day=5): Holiday.labour_day.value,
    datetime.date(year=2020, month=6, day=25): Holiday.dragon_boat_festival.value,
    datetime.date(year=2020, month=6, day=26): Holiday.dragon_boat_festival.value,
    datetime.date(year=2020, month=6, day=27): Holiday.dragon_boat_festival.value,
    datetime.date(year=2020, month=10, day=1): Holiday.national_day.value,
    datetime.date(year=2020, month=10, day=2): Holiday.national_day.value,
    datetime.date(year=2020, month=10, day=3): Holiday.national_day.value,
    datetime.date(year=2020, month=10, day=4): Holiday.national_day.value,
    datetime.date(year=2020, month=10, day=5): Holiday.national_day.value,
    datetime.date(year=2020, month=10, day=6): Holiday.national_day.value,
    datetime.date(year=2020, month=10, day=7): Holiday.national_day.value,
    datetime.date(year=2020, month=10, day=8): Holiday.national_day.value,
    datetime.date(year=2021, month=1, day=1): Holiday.new_years_day.value,
    datetime.date(year=2021, month=1, day=2): Holiday.new_years_day.value,
    datetime.date(year=2021, month=1, day=3): Holiday.new_years_day.value,
    datetime.date(year=2021, month=2, day=11): Holiday.spring_festival.value,
    datetime.date(year=2021, month=2, day=12): Holiday.spring_festival.value,
    datetime.date(year=2021, month=2, day=13): Holiday.spring_festival.value,
    datetime.date(year=2021, month=2, day=14): Holiday.spring_festival.value,
    datetime.date(year=2021, month=2, day=15): Holiday.spring_festival.value,
    datetime.date(year=2021, month=2, day=16): Holiday.spring_festival.value,
    datetime.date(year=2021, month=2, day=17): Holiday.spring_festival.value,
    datetime.date(year=2021, month=4, day=3): Holiday.tomb_sweeping_day.value,
    datetime.date(year=2021, month=4, day=4): Holiday.tomb_sweeping_day.value,
    datetime.date(year=2021, month=4, day=5): Holiday.tomb_sweeping_day.value,
    datetime.date(year=2021, month=5, day=1): Holiday.labour_day.value,
    datetime.date(year=2021, month=5, day=2): Holiday.labour_day.value,
    datetime.date(year=2021, month=5, day=3): Holiday.labour_day.value,
    datetime.date(year=2021, month=5, day=4): Holiday.labour_day.value,
    datetime.date(year=2021, month=5, day=5): Holiday.labour_day.value,
    datetime.date(year=2021, month=6, day=12): Holiday.dragon_boat_festival.value,
    datetime.date(year=2021, month=6, day=13): Holiday.dragon_boat_festival.value,
    datetime.date(year=2021, month=6, day=14): Holiday.dragon_boat_festival.value,
    datetime.date(year=2021, month=9, day=19): Holiday.mid_autumn_festival.value,
    datetime.date(year=2021, month=9, day=20): Holiday.mid_autumn_festival.value,
    datetime.date(year=2021, month=9, day=21): Holiday.mid_autumn_festival.value,
    datetime.date(year=2021, month=10, day=1): Holiday.national_day.value,
    datetime.date(year=2021, month=10, day=2): Holiday.national_day.value,
    datetime.date(year=2021, month=10, day=3): Holiday.national_day.value,
    datetime.date(year=2021, month=10, day=4): Holiday.national_day.value,
    datetime.date(year=2021, month=10, day=5): Holiday.national_day.value,
    datetime.date(year=2021, month=10, day=6): Holiday.national_day.value,
    datetime.date(year=2021, month=10, day=7): Holiday.national_day.value,
    datetime.date(year=2022, month=1, day=1): Holiday.new_years_day.value,
    datetime.date(year=2022, month=1, day=2): Holiday.new_years_day.value,
    datetime.date(year=2022, month=1, day=3): Holiday.new_years_day.value,
    datetime.date(year=2022, month=1, day=31): Holiday.spring_festival.value,
    datetime.date(year=2022, month=2, day=1): Holiday.spring_festival.value,
    datetime.date(year=2022, month=2, day=2): Holiday.spring_festival.value,
    datetime.date(year=2022, month=2, day=3): Holiday.spring_festival.value,
    datetime.date(year=2022, month=2, day=4): Holiday.spring_festival.value,
    datetime.date(year=2022, month=2, day=5): Holiday.spring_festival.value,
    datetime.date(year=2022, month=2, day=6): Holiday.spring_festival.value,
    datetime.date(year=2022, month=4, day=3): Holiday.tomb_sweeping_day.value,
    datetime.date(year=2022, month=4, day=4): Holiday.tomb_sweeping_day.value,
    datetime.date(year=2022, month=4, day=5): Holiday.tomb_sweeping_day.value,
    datetime.date(year=2022, month=4, day=30): Holiday.labour_day.value,
    datetime.date(year=2022, month=5, day=1): Holiday.labour_day.value,
    datetime.date(year=2022, month=5, day=2): Holiday.labour_day.value,
    datetime.date(year=2022, month=5, day=3): Holiday.labour_day.value,
    datetime.date(year=2022, month=5, day=4): Holiday.labour_day.value,
    datetime.date(year=2022, month=6, day=3): Holiday.dragon_boat_festival.value,
    datetime.date(year=2022, month=6, day=4): Holiday.dragon_boat_festival.value,
    datetime.date(year=2022, month=6, day=5): Holiday.dragon_boat_festival.value,
    datetime.date(year=2022, month=9, day=10): Holiday.mid_autumn_festival.value,
    datetime.date(year=2022, month=9, day=11): Holiday.mid_autumn_festival.value,
    datetime.date(year=2022, month=9, day=12): Holiday.mid_autumn_festival.value,
    datetime.date(year=2022, month=10, day=1): Holiday.national_day.value,
    datetime.date(year=2022, month=10, day=2): Holiday.national_day.value,
    datetime.date(year=2022, month=10, day=3): Holiday.national_day.value,
    datetime.date(year=2022, month=10, day=4): Holiday.national_day.value,
    datetime.date(year=2022, month=10, day=5): Holiday.national_day.value,
    datetime.date(year=2022, month=10, day=6): Holiday.national_day.value,
    datetime.date(year=2022, month=10, day=7): Holiday.national_day.value,
    datetime.date(year=2022, month=12, day=31): Holiday.new_years_day.value,
    datetime.date(year=2023, month=1, day=1): Holiday.new_years_day.value,
    datetime.date(year=2023, month=1, day=2): Holiday.new_years_day.value,
    datetime.date(year=2023, month=1, day=21): Holiday.spring_festival.value,
    datetime.date(year=2023, month=1, day=22): Holiday.spring_festival.value,
    datetime.date(year=2023, month=1, day=23): Holiday.spring_festival.value,
    datetime.date(year=2023, month=1, day=24): Holiday.spring_festival.value,
    datetime.date(year=2023, month=1, day=25): Holiday.spring_festival.value,
    datetime.date(year=2023, month=1, day=26): Holiday.spring_festival.value,
    datetime.date(year=2023, month=1, day=27): Holiday.spring_festival.value,
    datetime.date(year=2023, month=4, day=5): Holiday.tomb_sweeping_day.value,
    datetime.date(year=2023, month=4, day=29): Holiday.labour_day.value,
    datetime.date(year=2023, month=4, day=30): Holiday.labour_day.value,
    datetime.date(year=2023, month=5, day=1): Holiday.labour_day.value,
    datetime.date(year=2023, month=5, day=2): Holiday.labour_day.value,
    datetime.date(year=2023, month=5, day=3): Holiday.labour_day.value,
    datetime.date(year=2023, month=6, day=22): Holiday.dragon_boat_festival.value,
    datetime.date(year=2023, month=6, day=23): Holiday.dragon_boat_festival.value,
    datetime.date(year=2023, month=6, day=24): Holiday.dragon_boat_festival.value,
    datetime.date(year=2023, month=9, day=29): Holiday.mid_autumn_festival.value,
    datetime.date(year=2023, month=9, day=30): Holiday.national_day.value,
    datetime.date(year=2023, month=10, day=1): Holiday.national_day.value,
    datetime.date(year=2023, month=10, day=2): Holiday.national_day.value,
    datetime.date(year=2023, month=10, day=3): Holiday.national_day.value,
    datetime.date(year=2023, month=10, day=4): Holiday.national_day.value,
    datetime.date(year=2023, month=10, day=5): Holiday.national_day.value,
    datetime.date(year=2023, month=10, day=6): Holiday.national_day.value,
    datetime.date(year=2024, month=1, day=1): Holiday.new_years_day.value,
    datetime.date(year=2024, month=2, day=10): Holiday.spring_festival.value,
    datetime.date(year=2024, month=2, day=11): Holiday.spring_festival.value,
    datetime.date(year=2024, month=2, day=12): Holiday.spring_festival.value,
    datetime.date(year=2024, month=2, day=13): Holiday.spring_festival.value,
    datetime.date(year=2024, month=2, day=14): Holiday.spring_festival.value,
    datetime.date(year=2024, month=2, day=15): Holiday.spring_festival.value,
    datetime.date(year=2024, month=2, day=16): Holiday.spring_festival.value,
    datetime.date(year=2024, month=2, day=17): Holiday.spring_festival.value,
    datetime.date(year=2024, month=4, day=4): Holiday.tomb_sweeping_day.value,
    datetime.date(year=2024, month=4, day=5): Holiday.tomb_sweeping_day.value,
    datetime.date(year=2024, month=4, day=6): Holiday.tomb_sweeping_day.value,
    datetime.date(year=2024, month=5, day=1): Holiday.labour_day.value,
    datetime.date(year=2024, month=5, day=2): Holiday.labour_day.value,
    datetime.date(year=2024, month=5, day=3): Holiday.labour_day.value,
    datetime.date(year=2024, month=5, day=4): Holiday.labour_day.value,
    datetime.date(year=2024, month=5, day=5): Holiday.labour_day.value,
    datetime.date(year=2024, month=6, day=10): Holiday.dragon_boat_festival.value,
    datetime.date(year=2024, month=9, day=15): Holiday.mid_autumn_festival.value,
    datetime.date(year=2024, month=9, day=16): Holiday.mid_autumn_festival.value,
    datetime.date(year=2024, month=9, day=17): Holiday.mid_autumn_festival.value,
    datetime.date(year=2024, month=10, day=1): Holiday.national_day.value,
    datetime.date(year=2024, month=10, day=2): Holiday.national_day.value,
    datetime.date(year=2024, month=10, day=3): Holiday.national_day.value,
    datetime.date(year=2024, month=10, day=4): Holiday.national_day.value,
    datetime.date(year=2024, month=10, day=5): Holiday.national_day.value,
    datetime.date(year=2024, month=10, day=6): Holiday.national_day.value,
    datetime.date(year=2024, month=10, day=7): Holiday.national_day.value,
    datetime.date(year=2025, month=1, day=1): Holiday.new_years_day.value,
    datetime.date(year=2025, month=1, day=28): Holiday.spring_festival.value,
    datetime.date(year=2025, month=1, day=29): Holiday.spring_festival.value,
    datetime.date(year=2025, month=1, day=30): Holiday.spring_festival.value,
    datetime.date(year=2025, month=1, day=31): Holiday.spring_festival.value,
    datetime.date(year=2025, month=2, day=1): Holiday.spring_festival.value,
    datetime.date(year=2025, month=2, day=2): Holiday.spring_festival.value,
    datetime.date(year=2025, month=2, day=3): Holiday.spring_festival.value,
    datetime.date(year=2025, month=2, day=4): Holiday.spring_festival.value,
    datetime.date(year=2025, month=4, day=4): Holiday.tomb_sweeping_day.value,
    datetime.date(year=2025, month=4, day=5): Holiday.tomb_sweeping_day.value,
    datetime.date(year=2025, month=4, day=6): Holiday.tomb_sweeping_day.value,
    datetime.date(year=2025, month=5, day=1): Holiday.labour_day.value,
    datetime.date(year=2025, month=5, day=2): Holiday.labour_day.value,
    datetime.date(year=2025, month=5, day=3): Holiday.labour_day.value,
    datetime.date(year=2025, month=5, day=4): Holiday.labour_day.value,
    datetime.date(year=2025, month=5, day=5): Holiday.labour_day.value,
    datetime.date(year=2025, month=5, day=31): Holiday.dragon_boat_festival.value,
    datetime.date(year=2025, month=6, day=1): Holiday.dragon_boat_festival.value,
    datetime.date(year=2025, month=6, day=2): Holiday.dragon_boat_festival.value,
    datetime.date(year=2025, month=10, day=1): Holiday.national_day.value,
    datetime.date(year=2025, month=10, day=2): Holiday.national_day.value,
    datetime.date(year=2025, month=10, day=3): Holiday.national_day.value,
    datetime.date(year=2025, month=10, day=4): Holiday.national_day.value,
    datetime.date(year=2025, month=10, day=5): Holiday.national_day.value,
    datetime.date(year=2025, month=10, day=6): Holiday.mid_autumn_festival.value,
    datetime.date(year=2025, month=10, day=7): Holiday.national_day.value,
    datetime.date(year=2025, month=10, day=8): Holiday.national_day.value,
}

workdays = {
    datetime.date(year=2001, month=1, day=20): Holiday.spring_festival.value,
    datetime.date(year=2001, month=1, day=21): Holiday.spring_festival.value,
    datetime.date(year=2001, month=4, day=28): Holiday.labour_day.value,
    datetime.date(year=2001, month=4, day=29): Holiday.labour_day.value,
    datetime.date(year=2001, month=9, day=29): Holiday.national_day.value,
    datetime.date(year=2001, month=9, day=30): Holiday.national_day.value,
    datetime.date(year=2001, month=12, day=29): Holiday.new_years_day.value,
    datetime.date(year=2001, month=12, day=30): Holiday.new_years_day.value,
    datetime.date(year=2002, month=2, day=9): Holiday.spring_festival.value,
    datetime.date(year=2002, month=2, day=10): Holiday.spring_festival.value,
    datetime.date(year=2002, month=4, day=27): Holiday.labour_day.value,
    datetime.date(year=2002, month=4, day=28): Holiday.labour_day.value,
    datetime.date(year=2002, month=9, day=28): Holiday.national_day.value,
    datetime.date(year=2002, month=9, day=29): Holiday.national_day.value,
    datetime.date(year=2003, month=2, day=8): Holiday.spring_festival.value,
    datetime.date(year=2003, month=2, day=9): Holiday.spring_festival.value,
    datetime.date(year=2003, month=4, day=26): Holiday.labour_day.value,
    datetime.date(year=2003, month=4, day=27): Holiday.labour_day.value,
    datetime.date(year=2003, month=9, day=27): Holiday.national_day.value,
    datetime.date(year=2003, month=9, day=28): Holiday.national_day.value,
    datetime.date(year=2004, month=1, day=17): Holiday.spring_festival.value,
    datetime.date(year=2004, month=1, day=18): Holiday.spring_festival.value,
    datetime.date(year=2004, month=5, day=8): Holiday.labour_day.value,
    datetime.date(year=2004, month=5, day=9): Holiday.labour_day.value,
    datetime.date(year=2004, month=10, day=9): Holiday.national_day.value,
    datetime.date(year=2004, month=10, day=10): Holiday.national_day.value,
    datetime.date(year=2005, month=2, day=5): Holiday.spring_festival.value,
    datetime.date(year=2005, month=2, day=6): Holiday.spring_festival.value,
    datetime.date(year=2005, month=4, day=30): Holiday.labour_day.value,
    datetime.date(year=2005, month=5, day=8): Holiday.labour_day.value,
    datetime.date(year=2005, month=10, day=8): Holiday.national_day.value,
    datetime.date(year=2005, month=10, day=9): Holiday.national_day.value,
    datetime.date(year=2005, month=12, day=31): Holiday.new_years_day.value,
    datetime.date(year=2006, month=1, day=28): Holiday.spring_festival.value,
    datetime.date(year=2006, month=2, day=5): Holiday.spring_festival.value,
    datetime.date(year=2006, month=4, day=29): Holiday.labour_day.value,
    datetime.date(year=2006, month=4, day=30): Holiday.labour_day.value,
    datetime.date(year=2006, month=9, day=30): Holiday.national_day.value,
    datetime.date(year=2006, month=10, day=8): Holiday.national_day.value,
    datetime.date(year=2006, month=12, day=30): Holiday.new_years_day.value,
    datetime.date(year=2006, month=12, day=31): Holiday.new_years_day.value,
    datetime.date(year=2007, month=2, day=17): Holiday.spring_festival.value,
    datetime.date(year=2007, month=2, day=25): Holiday.spring_festival.value,
    datetime.date(year=2007, month=4, day=28): Holiday.labour_day.value,
    datetime.date(year=2007, month=4, day=29): Holiday.labour_day.value,
    datetime.date(year=2007, month=9, day=29): Holiday.national_day.value,
    datetime.date(year=2007, month=9, day=30): Holiday.national_day.value,
    datetime.date(year=2007, month=12, day=29): Holiday.new_years_day.value,
    datetime.date(year=2008, month=2, day=2): Holiday.spring_festival.value,
    datetime.date(year=2008, month=2, day=3): Holiday.spring_festival.value,
    datetime.date(year=2008, month=5, day=4): Holiday.labour_day.value,
    datetime.date(year=2008, month=9, day=27): Holiday.national_day.value,
    datetime.date(year=2008, month=9, day=28): Holiday.national_day.value,
    datetime.date(year=2009, month=1, day=4): Holiday.new_years_day.value,
    datetime.date(year=2009, month=1, day=24): Holiday.spring_festival.value,
    datetime.date(year=2009, month=2, day=1): Holiday.spring_festival.value,
    datetime.date(year=2009, month=5, day=31): Holiday.dragon_boat_festival.value,
    datetime.date(year=2009, month=9, day=27): Holiday.national_day.value,
    datetime.date(year=2009, month=10, day=10): Holiday.national_day.value,
    datetime.date(year=2010, month=2, day=20): Holiday.spring_festival.value,
    datetime.date(year=2010, month=2, day=21): Holiday.spring_festival.value,
    datetime.date(year=2010, month=6, day=12): Holiday.dragon_boat_festival.value,
    datetime.date(year=2010, month=6, day=13): Holiday.dragon_boat_festival.value,
    datetime.date(year=2010, month=9, day=19): Holiday.mid_autumn_festival.value,
    datetime.date(year=2010, month=9, day=25): Holiday.mid_autumn_festival.value,
    datetime.date(year=2010, month=9, day=26): Holiday.national_day.value,
    datetime.date(year=2010, month=10, day=9): Holiday.national_day.value,
    datetime.date(year=2011, month=1, day=30): Holiday.spring_festival.value,
    datetime.date(year=2011, month=2, day=12): Holiday.spring_festival.value,
    datetime.date(year=2011, month=4, day=2): Holiday.tomb_sweeping_day.value,
    datetime.date(year=2011, month=10, day=8): Holiday.national_day.value,
    datetime.date(year=2011, month=10, day=9): Holiday.national_day.value,
    datetime.date(year=2011, month=12, day=31): Holiday.new_years_day.value,
    datetime.date(year=2012, month=1, day=21): Holiday.spring_festival.value,
    datetime.date(year=2012, month=1, day=29): Holiday.spring_festival.value,
    datetime.date(year=2012, month=3, day=31): Holiday.tomb_sweeping_day.value,
    datetime.date(year=2012, month=4, day=1): Holiday.tomb_sweeping_day.value,
    datetime.date(year=2012, month=4, day=28): Holiday.labour_day.value,
    datetime.date(year=2012, month=9, day=29): Holiday.national_day.value,
    datetime.date(year=2013, month=1, day=5): Holiday.new_years_day.value,
    datetime.date(year=2013, month=1, day=6): Holiday.new_years_day.value,
    datetime.date(year=2013, month=2, day=16): Holiday.spring_festival.value,
    datetime.date(year=2013, month=2, day=17): Holiday.spring_festival.value,
    datetime.date(year=2013, month=4, day=7): Holiday.tomb_sweeping_day.value,
    datetime.date(year=2013, month=4, day=27): Holiday.labour_day.value,
    datetime.date(year=2013, month=4, day=28): Holiday.labour_day.value,
    datetime.date(year=2013, month=6, day=8): Holiday.dragon_boat_festival.value,
    datetime.date(year=2013, month=6, day=9): Holiday.dragon_boat_festival.value,
    datetime.date(year=2013, month=9, day=22): Holiday.mid_autumn_festival.value,
    datetime.date(year=2013, month=9, day=29): Holiday.national_day.value,
    datetime.date(year=2013, month=10, day=12): Holiday.national_day.value,
    datetime.date(year=2014, month=1, day=26): Holiday.spring_festival.value,
    datetime.date(year=2014, month=2, day=8): Holiday.spring_festival.value,
    datetime.date(year=2014, month=5, day=4): Holiday.labour_day.value,
    datetime.date(year=2014, month=9, day=28): Holiday.national_day.value,
    datetime.date(year=2014, month=10, day=11): Holiday.national_day.value,
    datetime.date(year=2015, month=1, day=4): Holiday.new_years_day.value,
    datetime.date(year=2015, month=2, day=15): Holiday.spring_festival.value,
    datetime.date(year=2015, month=2, day=28): Holiday.spring_festival.value,
    datetime.date(year=2015, month=9, day=6): Holiday.anti_fascist_70th_day.value,
    datetime.date(year=2015, month=10, day=10): Holiday.national_day.value,
    datetime.date(year=2016, month=2, day=6): Holiday.spring_festival.value,
    datetime.date(year=2016, month=2, day=14): Holiday.spring_festival.value,
    datetime.date(year=2016, month=6, day=12): Holiday.dragon_boat_festival.value,
    datetime.date(year=2016, month=9, day=18): Holiday.mid_autumn_festival.value,
    datetime.date(year=2016, month=10, day=8): Holiday.national_day.value,
    datetime.date(year=2016, month=10, day=9): Holiday.national_day.value,
    datetime.date(year=2017, month=1, day=22): Holiday.spring_festival.value,
    datetime.date(year=2017, month=2, day=4): Holiday.spring_festival.value,
    datetime.date(year=2017, month=4, day=1): Holiday.tomb_sweeping_day.value,
    datetime.date(year=2017, month=5, day=27): Holiday.dragon_boat_festival.value,
    datetime.date(year=2017, month=9, day=30): Holiday.national_day.value,
    datetime.date(year=2018, month=2, day=11): Holiday.spring_festival.value,
    datetime.date(year=2018, month=2, day=24): Holiday.spring_festival.value,
    datetime.date(year=2018, month=4, day=8): Holiday.tomb_sweeping_day.value,
    datetime.date(year=2018, month=4, day=28): Holiday.labour_day.value,
    datetime.date(year=2018, month=9, day=29): Holiday.national_day.value,
    datetime.date(year=2018, month=9, day=30): Holiday.national_day.value,
    datetime.date(year=2018, month=12, day=29): Holiday.new_years_day.value,
    datetime.date(year=2019, month=2, day=2): Holiday.spring_festival.value,
    datetime.date(year=2019, month=2, day=3): Holiday.spring_festival.value,
    datetime.date(year=2019, month=4, day=28): Holiday.labour_day.value,
    datetime.date(year=2019, month=5, day=5): Holiday.labour_day.value,
    datetime.date(year=2019, month=9, day=29): Holiday.national_day.value,
    datetime.date(year=2019, month=10, day=12): Holiday.national_day.value,
    datetime.date(year=2020, month=1, day=19): Holiday.spring_festival.value,
    datetime.date(year=2020, month=4, day=26): Holiday.labour_day.value,
    datetime.date(year=2020, month=5, day=9): Holiday.labour_day.value,
    datetime.date(year=2020, month=6, day=28): Holiday.dragon_boat_festival.value,
    datetime.date(year=2020, month=9, day=27): Holiday.national_day.value,
    datetime.date(year=2020, month=10, day=10): Holiday.national_day.value,
    datetime.date(year=2021, month=2, day=7): Holiday.spring_festival.value,
    datetime.date(year=2021, month=2, day=20): Holiday.spring_festival.value,
    datetime.date(year=2021, month=4, day=25): Holiday.labour_day.value,
    datetime.date(year=2021, month=5, day=8): Holiday.labour_day.value,
    datetime.date(year=2021, month=9, day=18): Holiday.mid_autumn_festival.value,
    datetime.date(year=2021, month=9, day=26): Holiday.national_day.value,
    datetime.date(year=2021, month=10, day=9): Holiday.national_day.value,
    datetime.date(year=2022, month=1, day=29): Holiday.spring_festival.value,
    datetime.date(year=2022, month=1, day=30): Holiday.spring_festival.value,
    datetime.date(year=2022, month=4, day=2): Holiday.tomb_sweeping_day.value,
    datetime.date(year=2022, month=4, day=24): Holiday.labour_day.value,
    datetime.date(year=2022, month=5, day=7): Holiday.labour_day.value,
    datetime.date(year=2022, month=10, day=8): Holiday.national_day.value,
    datetime.date(year=2022, month=10, day=9): Holiday.national_day.value,
    datetime.date(year=2023, month=1, day=28): Holiday.spring_festival.value,
    datetime.date(year=2023, month=1, day=29): Holiday.spring_festival.value,
    datetime.date(year=2023, month=4, day=23): Holiday.labour_day.value,
    datetime.date(year=2023, month=5, day=6): Holiday.labour_day.value,
    datetime.date(year=2023, month=6, day=25): Holiday.dragon_boat_festival.value,
    datetime.date(year=2023, month=10, day=7): Holiday.national_day.value,
    datetime.date(year=2023, month=10, day=8): Holiday.national_day.value,
    datetime.date(year=2024, month=2, day=4): Holiday.spring_festival.value,
    datetime.date(year=2024, month=2, day=18): Holiday.spring_festival.value,
    datetime.date(year=2024, month=4, day=7): Holiday.tomb_sweeping_day.value,
    datetime.date(year=2024, month=4, day=28): Holiday.labour_day.value,
    datetime.date(year=2024, month=5, day=11): Holiday.labour_day.value,
    datetime.date(year=2024, month=9, day=14): Holiday.mid_autumn_festival.value,
    datetime.date(year=2024, month=9, day=29): Holiday.national_day.value,
    datetime.date(year=2024, month=10, day=12): Holiday.national_day.value,
    datetime.date(year=2025, month=1, day=26): Holiday.spring_festival.value,
    datetime.date(year=2025, month=2, day=8): Holiday.spring_festival.value,
    datetime.date(year=2025, month=4, day=27): Holiday.labour_day.value,
    datetime.date(year=2025, month=9, day=28): Holiday.national_day.value,
    datetime.date(year=2025, month=10, day=11): Holiday.national_day.value,
}



# -*- coding: utf-8 -*-

import datetime
try:
    from .utils import trans_date, copy_format_from_start
    from .utils import get_file_path, load_yml
    from .constants.holiday import holidays, workdays
except:
    from chncal.utils import trans_date, copy_format_from_start
    from chncal.utils import get_file_path, load_yml
    from chncal.constants.holiday import holidays, workdays
    
    
def _validate_date(*dates):
    """check if the date(s) is supported"""
    if len(dates) != 1:
        return list(map(_validate_date, dates))
    date = trans_date(dates[0])
    if not isinstance(date, datetime.date):
        raise NotImplementedError('unsupported type {}, expected type is datetime.date'.format(type(date)))
    min_year, max_year = min(holidays.keys()).year, max(holidays.keys()).year
    if not (min_year <= date.year <= max_year):
        raise NotImplementedError(
            'no available data for year {}, only year between [{}, {}] supported'.format(date.year, min_year, max_year)
        )
    return date


@copy_format_from_start
def get_dates(start=None, end=None, copy_format=True):
    """
    get dates between start date and end date. (includes start date and end date)
    """
    start, end = map(trans_date, (start, end))
    delta_days = (end - start).days
    return [start + datetime.timedelta(days=delta) for delta in range(delta_days+1)]


def is_workday(date=None):
    """
    check if one date is workday in China.
    in other words, Chinese people works at that day.
    """
    date = _validate_date(date)
    weekday = date.weekday()
    return bool(date in workdays.keys() or (weekday <= 4 and date not in holidays.keys()))


def is_holiday(date=None):
    """
    check if one date is holiday in China.
    in other words, Chinese people get rest at that day.
    """
    return not is_workday(date)


@copy_format_from_start
def get_holidays(start=None, end=None, include_weekends=True, **kwargs):
    """
    get holidays between start date and end date. (includes start date and end date)
    """
    start, end = _validate_date(start, end)
    if include_weekends:
        return list(filter(is_holiday, get_dates(start, end, copy_format=False)))
    return list(filter(lambda x: x in holidays, get_dates(start, end, copy_format=False)))


@copy_format_from_start
def get_workdays(start=None, end=None, **kwargs):
    """
    get workdays between start date and end date. (includes start date and end date)
    """
    start, end = _validate_date(start, end)
    dates = get_dates(start, end, copy_format=False)
    return list(filter(is_workday, dates))


def get_work_dates(start_date=None, end_date=None, **kwargs):
    return get_workdays(start_date, end_date, **kwargs)


@copy_format_from_start
def _find_workday(date=None, delta_days=0, **kwargs):
    date = trans_date(date or datetime.date.today())
    if delta_days >= 0 and is_workday(date):
        delta_days += 1
    sign = 1 if delta_days >= 0 else -1
    for i in range(abs(delta_days)):
        if delta_days < 0 or i:
            date += datetime.timedelta(days=sign)
        while not is_workday(date):
            date += datetime.timedelta(days=sign)
    return date


def find_workday(delta_days=0, date=None):
    """
    find the workday after {delta_days} days. 查找date之后的第delta_days个工作日
    """
    return _find_workday(date=date, delta_days=delta_days)


@copy_format_from_start
def get_next_nth_workday(date=None, n=1, **kwargs):
    """
    给定日期date，返回其后第n个工作日日期，n可为负数（返回结果在date之前）
    若n为0，直接返回date
    """
    date = trans_date(date)
    n_add = -1 if n < 0 else 1
    n = abs(n)
    tmp = 0
    while tmp < n:
        date = date + datetime.timedelta(n_add)
        if is_workday(date):
            tmp += 1
    return date


@copy_format_from_start
def get_recent_workday(date=None, dirt='post', **kwargs):
    """
    若date为工作日，则直接返回，否则返回下一个(dirt为post时)或上一个(dirt为pre时)工作日
    """
    date = trans_date(date)
    tdelta = datetime.timedelta(1)
    if dirt == 'post':
        while not is_workday(date):
            date =  date + tdelta
    elif dirt == 'pre':
        while not is_workday(date):
            date =  date - tdelta
    return date


def get_holiday_detail(date=None):
    """
    check if one date is holiday in China,
    and return the holiday name (None if it's a normal day)
    """
    date = _validate_date(date)
    if date in workdays.keys():
        return False, workdays[date]
    elif date in holidays.keys():
        return True, holidays[date]
    else:
        return date.weekday() > 4, None
    
    
def get_holiday_info_source(year):
    data = load_yml(get_file_path('holiday.yml', 'data'))
    return data[int(year)]['info_from']



# -*- coding: utf-8 -*-

# 生肖合婚
# http://www.ibazi.cn/article/39858
# https://www.ddnx.com/shenghuo/242712.html


from __future__ import absolute_import, unicode_literals


MARRY_MATCH = {
    '鼠': {
        '宜': (('龙', '猴', '牛'), '大吉，心心相印，富贵幸福，万事易成功，终身。其他属相次之。'),
        '忌': (('马', '兔', '羊'), '不能富家，灾害并至，凶煞重重，甚至骨肉分离，不得安宁。'),
        '解释': '子鼠与丑牛六合，因此最宜找个属牛的对象，此乃上上等婚配。其次是与申猴辰龙三合，故也宜找个属猴或属龙的，此乃上等婚配。子鼠与午马相冲，因此最忌找个属马的，此乃下下等婚配。子鼠又与卯兔相刑，故也忌找属兔的，此乃下等婚配。子鼠与未羊也有相害的成分，故也不宜找属羊的，此乃中下等婚配。有时也讲子子自刑，此乃中下等婚配，故也应避免同属相的。'
    }, 
    '牛': {
        '宜': (('鼠', '蛇', '鸡'), '大吉，天做良缘，家道大旺，财盛人兴。'),
        '忌': (('马', '羊', '狗'), '吉凶各有，甘苦共存，无进取心，内心多忧疑苦惨。'),
        '解释': '丑牛与子鼠六合，因此最宜找个属鼠的对象，此乃上上等婚配。其次是与巳蛇酉鸡三合，故也宜找个属蛇或属鸡的，此乃上等婚配。丑牛与未羊相冲，因此最忌找属羊的，此乃下下等婚配。丑牛又与未羊、戌狗构成三刑，故最忌找属羊属狗的，此乃下等婚配。丑牛又与午马相害，故也不宜找属马的，此乃中下等婚配。'
    },
    '虎': {
        '宜': (('马', '狗', '猪'), '大吉，同心永结，德高望重，家业终成，富贵荣华，子孙昌盛。'),
        '忌': (('蛇', '猴'), '夫妻不和，忧愁不断，有破财之兆，空虚寂寞。'),
        '解释': '寅虎与亥猪六合，因此最宜找个属猪的对象，此乃上上等婚配。其次是与午马戌狗三合，此乃上等婚配，故也宜找个属马或属狗的。寅虎与申猴相冲，因此最忌找属猴的，此乃下下等婚配。寅虎又与巳蛇、申猴构成三刑，故最忌找属蛇属猴的，此乃下等婚配。'
    },
    '兔': {
        '宜': (('羊', '狗', '猪'), '功业成就，安居乐业，专利兴家。'),
        '忌': (('龙', '鼠', '鸡'), '家庭难有幸福，逆境之象。'),
        '解释': '卯兔与戌狗六合，因此最宜找个属狗的对象，此乃上上等婚配。其次是与亥猪未羊三合，故也宜找个属猪或属羊的，此乃上等婚配。卯兔与酉鸡相冲，因此最忌找属鸡的，此乃下下等婚配。卯兔与子鼠又有相刑的成分，故也不宜找属鼠的，此乃中下等婚配。卯兔还与辰龙相害，故也不宜找属龙的，此乃中下等婚配。'
    },
    '龙': {
        '宜': (('鼠', '猴', '鸡'), '大吉绨结良缘，勤俭发家，日刡昌盛，富贵成功，子孙继世。'),
        '忌': (('狗'), '不能和睦终世，破坏离别，不得心安。'),
        '解释': '辰龙与酉鸡六合，因此最宜找个属鸡的对象，此乃上上等婚配。其次是与申猴子鼠三合，故也宜找个属猴属鼠的，此乃上等婚配。辰龙与戌狗相冲，因此最忌找属狗的，此乃下下等婚配。辰龙与卯兔又有相害的成分，故也不宜找属兔的，此乃中下等婚配。有时也讲辰辰自刑，故也要注意避免同属相的，此乃中下等婚配。'
    },
    '蛇': {
        '宜': (('猴', '牛', '鸡'), '大吉祥，此中属相相配为福禄鸳鸯，智勇双全，功业垂成，足立宝地，名利双收，一生幸福。'),
        '忌': (('猪', '虎'), '家境虽无大的困苦和失败，但夫妻离心离德，子息缺少，灾厄百端，晚景不祥。'),
        '解释': '巳蛇与申猴六合，因此最宜找个属猴的对象，此乃上上等婚配。其次是与酉鸡丑牛三合，故也宜找个属鸡或属牛的，此乃上等婚配。巳蛇与亥猪相冲，因此最忌找属猪的，此乃下下等婚配。巳蛇与寅虎相刑，此乃下等婚配;有时也讲寅、巳、申三刑，故三口人不宜构成蛇、虎、猴的格局。'
    },
    '马': {
        '宜': (('虎', '羊', '狗'), '大吉，夫妻相敬，紫气东来，福乐安详，家道昌隆。'),
        '忌': (('鼠', '牛'), '中年运气尚可，病弱短寿，难望幸福，重生凶兆，一生辛苦，配偶早丧，子女别离。'),
        '解释': '午马与未羊六合，因此最宜找个属羊的对象，此乃上上等婚配。其次是与寅虎戌狗三合，故也宜找个属虎属猴的，此乃上等婚配。午马与子鼠相冲，因此最忌找属鼠的，此乃下下等婚配。午马与丑牛相害，因此也应避免找属牛的，此乃中下等婚配。有时又讲午午自刑，故也应避免同属相的，此乃中下等婚配。'
    },
    '羊': {
        '宜': (('兔', '马', '猪'), '天赐良缘，家道谐和，大业成而有德望。'),
        '忌': (('牛', '狗', '羊'), '夫妻一生难得幸福，多灾多难，一生劳碌，早失配偶或子孙。'),
        '解释': '未羊与午马六合，因此最宜找个属马的对象，此乃上上等婚配。其次是与亥猪卯兔三合，故也宜找个属猪属兔的，此乃上等婚配。未羊与丑牛相冲，因此最忌找属牛的，此乃下下等婚配。未羊与丑牛戌狗构成三刑，因此也不宜找属狗的，此乃下等婚配。未羊与子鼠相害，因此也不宜找属鼠的，此乃中下等婚配。'
    },
    '猴': {
        '宜': (('鼠', '蛇', '龙'), '此中属相相配为珠联璧合，一帆风顺，富贵成功，子孙兴旺。'),
        '忌': (('虎', '猪'), '灾害多起，晚景尚可，但恐寿不到永，疾病困难。'),
        '解释': '申猴与巳蛇六合，因此最宜找个属蛇的对象，此乃上上等婚配。其次是与子鼠辰龙三合，故也宜找个属鼠属龙的，此乃上等婚配。申猴与寅虎相冲，因此最忌找属虎的，此乃下下等婚配。申猴又与巳蛇寅虎构成三刑，因此三口人之间不宜形成虎、蛇、猴这种配合。申猴又与亥猪相害，因此也不宜找属猪的，此乃中下等婚配。'
    },
    '鸡': {
        '宜': (('牛', '龙', '蛇'), '此中属相相配祥开白事，有天赐之福，并有名望，功利荣达，家事亨通。'),
        '忌': (('兔', '狗'), '金鸡玉犬难逃避，合婚双份不可迁，多灾多难。'),
        '解释': '酉鸡与辰龙六合，因此最宜找个属龙的对象，此乃上上等婚配。其次是与巳蛇丑牛三合，故也宜找个属蛇属牛的，此乃上等婚配。酉鸡与卯兔相冲，因此最忌找属兔的，此乃下下等婚配。酉鸡又与戌狗相害，故也不宜找属狗的，此乃中下等婚配。' 
    },
    '狗': {
        '宜': (('虎', '兔', '马'), '大吉，天做之合，处处成功，福碌永久，家运昌隆。'),
        '忌': (('羊', '龙', '鸡', '牛'), '灾害垒起，钱财散败，一生艰辛，事与愿违。'),
        '解释': '戌狗与卯兔六合，因此最宜找个属兔的对象，此乃上上等婚配。其次是与寅虎午马三合，故也宜找个属虎属马的，此乃上等婚配。戌狗与辰龙相冲，故最忌找属龙的，此乃下下等婚配。戌狗又与未羊丑牛构成三刑，故不宜找属羊属牛的，此乃下等婚配。戌狗又与酉鸡相害，故不宜找属鸡的，此乃中下等婚配。'
    },
    '猪': {
        '宜': (('羊', '兔', '虎'), '大吉，五事其昌，安富尊荣，子孙健壮，积财多福。'),
        '忌': (('猴', '蛇'), '猪猴不到头，朝朝日日泪交流，比能共长久，终生难于幸福。'),
        '解释': '亥猪与寅虎六合，因此最宜找个属虎的对象，此乃上上等婚配。其次是与卯兔未羊三合，故也宜找个属兔属羊的，此乃上等婚配。亥猪与巳蛇相冲，故最忌找属蛇的，此乃下下等婚配。亥猪又与申猴相害，因此也不宜找属猴的，此乃中下等婚配。有时也讲亥亥自刑，故也不宜找同属相的，此乃中下等婚配。',
    }
}



# -*- coding: utf-8 -*-

import datetime
import pandas as pd
try:
    from .utils import trans_date, copy_format_from_start
    from .constants.trade_dates import MARKETS, trade_dates
    from .holiday import is_workday
except:
    from chncal.utils import trans_date, copy_format_from_start
    from chncal.holiday import is_workday
    from chncal.constants.trade_dates import MARKETS, trade_dates


def _is_tradeday(date=None):
    return is_workday(date) and date.weekday() not in [5, 6]


def is_tradeday(date=None, market='SSE'):
    """判断是否为交易日"""
    market = market.upper()
    assert market in MARKETS, '未识别的交易所，请检查！'
    date = trans_date(date)    
    if date < MARKETS[market]:
        return False  # 小于首个交易日的直接视为非交易日
    if (market, date) in trade_dates:
        return bool(trade_dates[(market, date)])
    return _is_tradeday(date)


@copy_format_from_start
def get_recent_tradeday(date=None, dirt='post', market='SSE'):
    """
    查询最近的交易日，若date为交易日，则直接返回date，否则返回下一个(dirt='post')或上一个(dirt='pre')交易日
    """
    assert dirt in ['post', 'pre']
    date = trans_date(date)
    tdelta = datetime.timedelta(1)
    if dirt == 'post':
        while not is_tradeday(date, market=market):
            date = date + tdelta
    elif dirt == 'pre':
        while not is_tradeday(date, market=market):
            if date.date() < MARKETS[market]:
                date = MARKETS[market]
                break
            date = date - tdelta
    return date


@copy_format_from_start
def get_next_nth_tradeday(date=None, n=1, market='SSE'):
    """
    | 给定日期date，返回其后第n个交易日日期，n可为负数（返回结果在date之前）
    | 若n为0，直接返回date
    """
    date = trans_date(date)
    n_add = -1 if n < 0 else 1
    n = abs(n)
    tmp = 0
    while tmp < n:
        if n_add == -1 and date.date() <= MARKETS[market]:
            break
        date = date + datetime.timedelta(n_add)
        if is_tradeday(date, market=market):
            tmp += 1
    return date


@copy_format_from_start
def get_trade_dates(start_date, end_date=None, market='SSE'):
    """
    取指定起止日期内的交易日期（周内的工作日）
    """
    start_date = trans_date(start_date)
    end_date = trans_date(end_date)
    dates = pd.date_range(start_date, end_date)
    dates = [x for x in dates if is_tradeday(x, market=market)]
    return dates



# -*- coding: utf-8 -*-

import sys

if 'bdist_wheel' in sys.argv:
    # to build wheel, use 'python setup.py sdist bdist_wheel'
    from setuptools import setup
else:
    from distutils.core import setup # 'python setup.py install'

with open('README.md', 'r', encoding='utf-8') as f:
    long_description = f.read()
    
__version__ = None
with open('./chncal/__init__.py', 'r', encoding='utf-8') as f:
    exec(f.readlines()[4].strip())

setup(
    name='chncal',
    version=__version__,
    description='Check if some day is holiday in china; Chinese lunar calendar.',
    long_description=long_description,
    long_description_content_type='text/markdown',
    author='Genlovy Hoo',
    author_email='genlovhyy@163.com',
    url='https://gitee.com/glhyy/ChnCal',
    license='MIT License',
    install_requires=['dramkit'],
    packages=['chncal',
              'chncal.constants'],
)



# * coding: utf8 *

import datetime
import pandas as pd

try:
    from .utils import trans_date
    from .constants.hko_calendar import sol_lun, lun_sol, sol_gz
except:
    from chncal.utils import trans_date
    from chncal.constants.hko_calendar import sol_lun, lun_sol, sol_gz
    

# 干支纪年https://baike.baidu.com/item/干支纪年/3383226
# 天干
TG = ['甲', '乙', '丙', '丁', '戊', '己', '庚', '辛', '壬', '癸']
# 地支
DZ = ['子', '丑', '寅', '卯', '辰', '巳', '午', '未', '申', '酉', '戌', '亥']
DZ_HOUR = {
    '子': (23, 1), '丑': (1, 3), '寅': (3, 5), '卯': (5, 7),
    '辰': (7, 9), '巳': (9, 11), '午': (11, 13), '未': (13, 15),
    '申': (15, 17), '酉': (17, 19), '戌': (19, 21), '亥': (21, 23)
}
# 生肖
SX = ['鼠', '牛', '虎', '兔', '龙', '蛇', '马', '羊', '猴', '鸡', '狗', '猪']
def _sol_tgdz():
    k1, k2 = 0, 0
    TGDZ = [TG[k1]+DZ[k2]+'('+SX[k2]+')']
    while not (k1 == len(TG)-1 and  k2 == len(DZ)-1):
        if k1 < len(TG)-1:
            k1 += 1
        else:
            k1 = 0
        if k2 < len(DZ)-1:
            k2 += 1
        else:
            k2 = 0
        TGDZ.append(TG[k1]+DZ[k2]+'('+SX[k2]+')')
    return TGDZ
TGDZ = _sol_tgdz()
TGDZ = ['甲子(鼠)', '乙丑(牛)', '丙寅(虎)', '丁卯(兔)', '戊辰(龙)', '己巳(蛇)',
        '庚午(马)', '辛未(羊)', '壬申(猴)', '癸酉(鸡)', '甲戌(狗)', '乙亥(猪)',
        '丙子(鼠)', '丁丑(牛)', '戊寅(虎)', '己卯(兔)', '庚辰(龙)', '辛巳(蛇)',
        '壬午(马)', '癸未(羊)', '甲申(猴)', '乙酉(鸡)', '丙戌(狗)', '丁亥(猪)',
        '戊子(鼠)', '己丑(牛)', '庚寅(虎)', '辛卯(兔)', '壬辰(龙)', '癸巳(蛇)',
        '甲午(马)', '乙未(羊)', '丙申(猴)', '丁酉(鸡)', '戊戌(狗)', '己亥(猪)',
        '庚子(鼠)', '辛丑(牛)', '壬寅(虎)', '癸卯(兔)', '甲辰(龙)', '乙巳(蛇)',
        '丙午(马)', '丁未(羊)', '戊申(猴)', '己酉(鸡)', '庚戌(狗)', '辛亥(猪)',
        '壬子(鼠)', '癸丑(牛)', '甲寅(虎)', '乙卯(兔)', '丙辰(龙)', '丁巳(蛇)',
        '戊午(马)', '己未(羊)', '庚申(猴)', '辛酉(鸡)', '壬戌(狗)', '癸亥(猪)']
# 农历2022年六月十二（公历2022.07.10）是甲子日
TGDZ_BASE_DATE = datetime.date(2022, 7, 10)
# 公历2022.08.24凌晨是甲子时
# TGDZ_BASE_TIME = datetime.datetime(2022, 8, 23, 23, 17, 5)
TGDZ_BASE_TIME = datetime.datetime(2022, 8, 23, 0, 0)


def hour2dz(hour: int = None):
    """获取小时的地支，hour为0-23之间的整数"""
    if hour is None:
        hour = datetime.datetime.now().hour
    hour = int(hour)
    assert 0 <= hour <= 23
    dz = None
    for name, (start, end) in DZ_HOUR.items():
        if hour >= start and hour < end:
            dz = name
    if dz is None:
        dz = '子'
    return dz
    
    
def sol2lun(date=None):
    """公历日期转农历日期"""
    return sol_lun[trans_date(date).strftime('%Y.%m.%d')]


def sol2gz(date=None):
    """公历日期转干支纪日法"""
    return sol_gz[trans_date(date).strftime('%Y.%m.%d')]


def lun2sol(date, run=False):
    """
    农历日期转普通日期，date格式必须形如'2023.02.30'，run为True表示闰月日期
    
    Examples
    --------
    >>> lun2sol('2023.02.30')
    """
    assert isinstance(date, str) and len(date) == 10 and '.' in date
    if run:
        date = date + '闰'
    if date in lun_sol:
        return lun_sol[date]
    else:
        raise ValueError('未找到对应农历日期，请检查输入！')
        
        
def get_tgdz_year(year : int=None):
    """计算年份天干地支"""
    if pd.isnull(year):
        year = datetime.datetime.now().year
    # 农历1984年是甲子年
    year = int(year)
    if year >= 1984:
        return TGDZ[(year-1984) % 60]
    else:
        return TGDZ[-((1984-year) % 60)]
    

def get_tgdz_date(date=None):
    """根据公历日期计算农历干支纪日"""
    date = trans_date(date)
    days = (date - TGDZ_BASE_DATE).days
    if days >= 0:
        return TGDZ[days % 60]
    else:
        return TGDZ[-(abs(days) % 60)]
    
    
def get_tgdz_hour(time=None):
    """
    根据公历时间（小时）计算农历干支纪时
    跟寿星天文历有误差（http://www.nongli.net/sxwnl/）
    """
    if pd.isnull(time):
        time = datetime.datetime.now().strftime('%Y%m%d %H:%M:%S')
    time = str(time)
    dif = pd.to_datetime(time) - TGDZ_BASE_TIME
    days = dif.days
    seconds = dif.seconds + days*24*3600
    hours2 = seconds / 7200
    if hours2 >= 0:
        return TGDZ[int(hours2 % 60)]
    else:
        return TGDZ[-(int(abs(hours2) % 60))-1]
